@@ -15,9 +15,7 @@
  */
 package com.milaboratory.core;
 
-import com.milaboratory.core.io.sequence.PairedRead;
-import com.milaboratory.core.io.sequence.SequenceRead;
-import com.milaboratory.core.io.sequence.SingleRead;
+import com.milaboratory.core.sequence.NSQTuple;
 import com.milaboratory.core.sequence.NSequenceWithQuality;
 
 public enum PairedEndReadsLayout implements java.io.Serializable {
@@ -106,20 +104,15 @@ public enum PairedEndReadsLayout implements java.io.Serializable {
         this.possibleRelativeStrands = possibleRelativeStrands;
     }
 
-    public Target[] createTargets(PairedRead read) {
-        return pairedProvider.createTargets(read);
-    }
-
-    public Target[] createTargets(SingleRead read) {
-        return singleProvider.createTargets(read);
-    }
-
-    public Target[] createTargets(SequenceRead read) {
-        if (read instanceof PairedRead)
-            return pairedProvider.createTargets((PairedRead) read);
-        if (read instanceof SingleRead)
-            return singleProvider.createTargets((SingleRead) read);
-        throw new IllegalArgumentException("Unknown read type.");
+    public Target[] createTargets(NSQTuple read) {
+        switch (read.size()) {
+            case 1:
+                return singleProvider.createTargets(read);
+            case 2:
+                return pairedProvider.createTargets(read);
+            default:
+                throw new IllegalArgumentException("Unknown read type.");
+        }
     }
 
     /**
@@ -138,13 +131,16 @@ public enum PairedEndReadsLayout implements java.io.Serializable {
             this.states = states;
         }
 
-        Target[] createTargets(SingleRead read) {
+        Target[] createTargets(NSQTuple input) {
+            assert input.size() == 1;
             Target[] ts = new Target[states.length];
             int i = 0;
             for (boolean state : states)
-                ts[i++] = new Target(state ?
-                        read.getData().getReverseComplement() :
-                        read.getData(), state);
+                ts[i++] = new Target(
+                        state ?
+                                input.get(0).getReverseComplement() :
+                                input.get(0),
+                        state);
             return ts;
         }
     }
@@ -160,26 +156,27 @@ public enum PairedEndReadsLayout implements java.io.Serializable {
             }
         }
 
-        Target[] createTargets(PairedRead read) {
+        Target[] createTargets(NSQTuple input) {
+            assert input.size() == 2;
             final Target[] result = new Target[ids.length];
             for (int i = 0; i < ids.length; i++) {
                 byte[] ii = ids[i];
-                result[i] = new Target(dataFromId(read, ii[0]),
-                        dataFromId(read, ii[1]), ii);
+                result[i] = new Target(dataFromId(input, ii[0]),
+                        dataFromId(input, ii[1]), ii);
             }
             return result;
         }
 
-        NSequenceWithQuality dataFromId(PairedRead read, byte id) {
+        NSequenceWithQuality dataFromId(NSQTuple read, byte id) {
             switch (id) {
                 case +1:
-                    return read.getR1().getData();
+                    return read.get(0);
                 case +2:
-                    return read.getR2().getData();
+                    return read.get(1);
                 case -1:
-                    return read.getR1().getData().getReverseComplement();
+                    return read.get(0).getReverseComplement();
                 case -2:
-                    return read.getR2().getData().getReverseComplement();
+                    return read.get(1).getReverseComplement();
             }
             throw new IllegalArgumentException();
         }
