@@ -1,5 +1,5 @@
 /*
- * Copyright 2015 MiLaboratory.com
+ * Copyright (c) 2022 MiLaboratories Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -128,6 +128,20 @@ public final class Range implements java.io.Serializable, Comparable<Range> {
         return new Range(newLower, upper, reversed);
     }
 
+    /**
+     * Returns a new range object with the edge values moved by the corresponding offset
+     *
+     * @param lowerValueOffset lower value offset
+     * @param upperValueOffset upper value offset
+     * @return new range with the edge values moved by the corresponding offset
+     */
+    public Range offset(int lowerValueOffset, int upperValueOffset) {
+        if (lowerValueOffset == 0 && upperValueOffset == 0)
+            return this;
+        return new Range(lower + lowerValueOffset, upper + upperValueOffset, reversed);
+    }
+
+
     /**
      * Return reversed range.
      *
@@ -283,6 +297,19 @@ public final class Range implements java.io.Serializable, Comparable<Range> {
         return new Range(Math.min(lower, other.lower), Math.max(upper, other.upper), reversed && other.reversed);
     }
 
+    /**
+     * Returns union range with {@code other} range, throws IllegalArgument exception if there is no intersection.
+     *
+     * @param other other range
+     * @return union range with {@code other} range
+     */
+    public Range merge(Range other) {
+        if (!intersectsWithOrTouches(other))
+            throw new IllegalArgumentException("Ranges don't intersect this = " + this + " ; other = " + other);
+
+        return new Range(Math.min(lower, other.lower), Math.max(upper, other.upper), reversed && other.reversed);
+    }
+
     /**
      * Returns range moved using provided offset (e.g. [lower + offset, upper + offset, reversed])
      *
@@ -290,9 +317,7 @@ public final class Range implements java.io.Serializable, Comparable<Range> {
      * @return range moved using provided offset
      */
     public Range move(int offset) {
-        if (offset == 0)
-            return this;
-        return new Range(lower + offset, upper + offset, reversed);
+        return offset(offset, offset);
     }
 
     /**
@@ -461,4 +486,16 @@ public final class Range implements java.io.Serializable, Comparable<Range> {
             return Integer.compare(o1.getFrom(), o2.getTo());
         }
     };
+
+    public static Range createFromOffsetAndLength(int offset, int length) {
+        return new Range(offset, offset + length);
+    }
+
+    public static Range naturalRange(int length) {
+        return new Range(0, length);
+    }
+
+    public static Range createFromAroundPoint(int point, int offsetFrom, int offsetTo) {
+        return new Range(point + offsetFrom, point + offsetTo);
+    }
 }
