@@ -1,5 +1,5 @@
 /*
- * Copyright 2015 MiLaboratory.com
+ * Copyright (c) 2022 MiLaboratories Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,8 +15,6 @@
  */
 package com.milaboratory.core.alignment;
 
-import cc.redberry.pipe.CUtils;
-import cc.redberry.pipe.OutputPort;
 import com.milaboratory.core.Range;
 import com.milaboratory.core.io.binary.AlignmentSerializer;
 import com.milaboratory.core.mutations.Mutation;
@@ -40,7 +38,7 @@ import static com.milaboratory.core.mutations.Mutation.*;
 @Serializable(by = AlignmentSerializer.class)
 public final class Alignment<S extends Sequence<S>> implements java.io.Serializable {
     /**
-     * Initial sequence. (upper sequence in alignment; sequence1)
+     * Initial sequence. (upper sequence in alignment; sequence1; reference)
      */
     final S sequence1;
     /**
@@ -48,11 +46,11 @@ public final class Alignment<S extends Sequence<S>> implements java.io.Serializa
      */
     final Mutations<S> mutations;
     /**
-     * Range in initial sequence (sequence1)
+     * Range in initial sequence (sequence1; reference)
      */
     final Range sequence1Range;
     /**
-     * Range in mutated sequence (sequence2)
+     * Range in mutated sequence (sequence2; query)
      */
     final Range sequence2Range;
     /**
@@ -232,8 +230,12 @@ public final class Alignment<S extends Sequence<S>> implements java.io.Serializa
      * @return range in sequence1 or null if rangeInSeq2 is not fully covered by alignment
      */
     public Range convertToSeq1Range(Range rangeInSeq2) {
-        int from = aabs(convertToSeq1Position(rangeInSeq2.getFrom()));
-        int to = aabs(convertToSeq1Position(rangeInSeq2.getTo()));
+        int from = rangeInSeq2.getFrom() == sequence2Range.getFrom()
+                ? sequence1Range.getFrom()
+                : aabs(convertToSeq1Position(rangeInSeq2.getFrom()));
+        int to = rangeInSeq2.getTo() == sequence2Range.getTo()
+                ? sequence1Range.getTo()
+                : aabs(convertToSeq1Position(rangeInSeq2.getTo()));
 
         if (from == -1 || to == -1)
             return null;
@@ -249,8 +251,12 @@ public final class Alignment<S extends Sequence<S>> implements java.io.Serializa
      * @return range in sequence2 or null if rangeInSeq1 is not fully covered by alignment
      */
     public Range convertToSeq2Range(Range rangeInSeq1) {
-        int from = aabs(convertToSeq2Position(rangeInSeq1.getFrom()));
-        int to = aabs(convertToSeq2Position(rangeInSeq1.getTo()));
+        int from = rangeInSeq1.getFrom() == sequence1Range.getFrom()
+                ? sequence2Range.getFrom()
+                : aabs(convertToSeq2Position(rangeInSeq1.getFrom()));
+        int to = rangeInSeq1.getTo() == sequence1Range.getTo()
+                ? sequence2Range.getTo()
+                : aabs(convertToSeq2Position(rangeInSeq1.getTo()));
 
         if (from == -1 || to == -1)
             return null;
@@ -367,23 +373,15 @@ public final class Alignment<S extends Sequence<S>> implements java.io.Serializa
         return new Alignment<>(sequence1, mutations, sequence1Range, sequence2Range.move(offset), score);
     }
 
-    /**
-     * Returns iterator over alignments elements in terms of CIGAR string.
-     *
-     * @return iterator over alignments elements in terms of CIGAR string
-     */
-    public OutputPort<AlignmentElement> getAlignmentElements() {
-        return new AlignmentElementOP();
-    }
-
     /**
      * Returns CIGAR string representation of this alignment.
      *
+     * @param full if true, skip operations to establish global query/reference positioning will be added
      * @return CIGAR string representation of this alignment
      */
-    public String getCigarString() {
+    public String getCigarString(boolean full) {
         StringBuilder sb = new StringBuilder();
-        for (AlignmentElement ae : CUtils.it(getAlignmentElements())) {
+        for (AlignmentElement ae : getAlignmentElements(full)) {
             sb.append(ae.getCigarLength());
             sb.append(ae.getType().cigarLetterUpper);
         }
@@ -434,80 +432,119 @@ public final class Alignment<S extends Sequence<S>> implements java.io.Serializa
         return position;
     }
 
-    private final class AlignmentElementOP implements OutputPort<AlignmentElement> {
+    /**
+     * Returns iterator over alignments elements in terms of CIGAR string.
+     *
+     * @param full if true, skip operations to establish global query/reference positioning will be added
+     * @return iterator over alignments elements in terms of CIGAR string
+     */
+    public List<AlignmentElement> getAlignmentElements(boolean full) {
+        List<AlignmentElement> result = new ArrayList<>();
+
+        // adding skip elements
+        if (sequence2Range.getLower() > 0)
+            result.add(new AlignmentElement(
+                    new Range(0, 0), new Range(0, sequence2Range.getLower()), false));
+        if (sequence1Range.getLower() > 0)
+            result.add(new AlignmentElement(
+                    new Range(0, sequence1Range.getLower()),
+                    new Range(sequence2Range.getLower(), sequence2Range.getLower()), false));
+
         int currentMutationIndex = -1;
         int lastSeq1Position = sequence1Range.getFrom();
         int lastSeq2Position = sequence2Range.getFrom();
 
-        AlignmentElementOP() {
-            if (sequence1Range.isReverse() || sequence2Range.isReverse())
-                throw new IllegalArgumentException();
-        }
-
-        @Override
-        public AlignmentElement take() {
-            int mutation;
-            boolean isMatch = true;
-            while (true) {
-                ++currentMutationIndex;
-
-                if (currentMutationIndex > mutations.size())
-                    return null;
-
-                if (currentMutationIndex == mutations.size()) {
-                    if (lastSeq1Position == sequence1Range.getTo()) {
-                        assert lastSeq2Position == sequence2Range.getTo();
-                        return null;
-                    }
-                    return new AlignmentElement(
-                            new Range(lastSeq1Position, sequence1Range.getTo()),
-                            new Range(lastSeq2Position, sequence2Range.getTo()),
-                            isMatch);
-                }
+        int mutation;
+        int newSeq1Position;
+        int matchedLetters;
+        int newSeq2Position;
+        while (true) {
+            ++currentMutationIndex;
 
+            if (currentMutationIndex == mutations.size()) { // all mutations processed
+                mutation = NON_MUTATION;
+                newSeq1Position = sequence1Range.getTo();
+            } else { // ingesting next mutation
                 mutation = mutations.getMutation(currentMutationIndex);
+                newSeq1Position = getPosition(mutation);
+            }
 
-                if (isSubstitution(mutation))
-                    isMatch = false;
+            matchedLetters = newSeq1Position - lastSeq1Position;
+            newSeq2Position = lastSeq2Position + matchedLetters;
 
-                if (isInDel(mutation)) {
-                    int newSeq1Position = getPosition(mutation);
-                    int newSeq2Position;
+            if (matchedLetters > 0) { // there are matching letters between current and previous mutations
+                result.add(new AlignmentElement(
+                        new Range(lastSeq1Position, newSeq1Position),
+                        new Range(lastSeq2Position, newSeq2Position),
+                        true));
+            }
 
-                    if (newSeq1Position != lastSeq1Position) {
-                        --currentMutationIndex; // This mutation will be inspected again on the next round
-                        newSeq2Position = lastSeq2Position + (newSeq1Position - lastSeq1Position);
-                    } else {
-                        // newSeq1Position == lastSeq1Position == getPosition(mutation)
-                        newSeq2Position = lastSeq2Position;
-                        if (isInsertion(mutation)) {
-                            ++newSeq2Position;
-                            while (currentMutationIndex + 1 < mutations.size() &&
-                                    isInsertion(mutation = mutations.getMutation(currentMutationIndex + 1)) &&
-                                    getPosition(mutation) == newSeq1Position) {
-                                ++currentMutationIndex;
-                                ++newSeq2Position;
-                            }
-                        } else {
-                            ++newSeq1Position;
-                            while (currentMutationIndex + 1 < mutations.size() &&
-                                    isDeletion(mutation = mutations.getMutation(currentMutationIndex + 1)) &&
-                                    getPosition(mutation) == newSeq1Position) {
-                                ++currentMutationIndex;
-                                ++newSeq1Position;
-                            }
-                        }
-                    }
+            if (mutation == NON_MUTATION) {
+                assert newSeq2Position == sequence2Range.getTo();
+                break;
+            }
 
-                    AlignmentElement ret = new AlignmentElement(
-                            new Range(lastSeq1Position, newSeq1Position),
-                            new Range(lastSeq2Position, newSeq2Position),
-                            isMatch);
-                    lastSeq1Position = newSeq1Position;
-                    lastSeq2Position = newSeq2Position;
-                    return ret;
-                }
+            lastSeq1Position = newSeq1Position;
+            lastSeq2Position = newSeq2Position;
+
+            if (isSubstitution(mutation)) {
+                // determine the last adjacent substitution in the group
+                do {
+                    ++newSeq1Position;
+                    ++newSeq2Position;
+                } while (++currentMutationIndex != mutations.size() &&
+                        isSubstitution(mutation = mutations.getMutation(currentMutationIndex)) &&
+                        getPosition(mutation) == newSeq1Position);
+
+                // previous loop always overruns by one letter
+                --currentMutationIndex;
+
+                // adding mismatch group
+                result.add(new AlignmentElement(
+                        new Range(lastSeq1Position, newSeq1Position),
+                        new Range(lastSeq2Position, newSeq2Position),
+                        false));
+
+                lastSeq1Position = newSeq1Position;
+                lastSeq2Position = newSeq2Position;
+            } else if (isInsertion(mutation)) {
+                // determine the last adjacent insertion in the group
+                do {
+                    ++newSeq2Position;
+                } while (++currentMutationIndex != mutations.size() &&
+                        isInsertion(mutation = mutations.getMutation(currentMutationIndex)) &&
+                        getPosition(mutation) == newSeq1Position);
+
+                // previous loop always overruns by one letter
+                --currentMutationIndex;
+
+                // adding mismatch group
+                result.add(new AlignmentElement(
+                        new Range(lastSeq1Position, newSeq1Position),
+                        new Range(lastSeq2Position, newSeq2Position),
+                        true));
+
+                lastSeq2Position = newSeq2Position;
+            } else { // deletion
+                // determine the last deletion substitution in the group
+                do {
+                    ++newSeq1Position;
+                } while (++currentMutationIndex != mutations.size() &&
+                        isDeletion(mutation = mutations.getMutation(currentMutationIndex)) &&
+                        getPosition(mutation) == newSeq1Position);
+
+                // previous loop always overruns by one letter
+                --currentMutationIndex;
+
+                // adding mismatch group
+                result.add(new AlignmentElement(
+                        new Range(lastSeq1Position, newSeq1Position),
+                        new Range(lastSeq2Position, newSeq2Position),
+                        true));
+
+                lastSeq1Position = newSeq1Position;
             }
         }
+        return result;
     }
 }
