@@ -21,6 +21,7 @@ import com.milaboratory.core.mutations.Mutations;
 import com.milaboratory.core.mutations.MutationsUtil;
 import com.milaboratory.core.sequence.Alphabet;
 import com.milaboratory.core.sequence.Sequence;
+import com.milaboratory.core.sequence.SequenceQuality;
 
 import static com.milaboratory.core.mutations.Mutation.*;
 
@@ -107,6 +108,72 @@ public final class AlignmentUtils {
         return score;
     }
 
+    /**
+     * Calculates score of alignment
+     *
+     * @param seq1        target sequence
+     * @param seq2Quality target sequence
+     * @param seq1Range   aligned range
+     * @param mutations   mutations (alignment)
+     * @param scoring     scoring
+     * @param <S>         sequence type
+     * @return score
+     */
+    public static <S extends Sequence<S>> int calculateQScore(S seq1,
+                                                              SequenceQuality seq2Quality,
+                                                              Range seq1Range, Range seq2Range,
+                                                              Mutations<S> mutations,
+                                                              LinearGapAlignmentScoring<S> scoring) {
+        if (!mutations.isEmpty() && mutations.getPositionByIndex(0) < seq1Range.getFrom() - 1)
+            throw new IllegalArgumentException();
+
+        final AlignmentIteratorForward<S> iterator = new AlignmentIteratorForward<>(mutations, seq1Range, seq2Range.getFrom());
+
+        int score = 0;
+
+        while (iterator.advance()) {
+            final int mut = iterator.getCurrentMutation();
+            int seq2Pos = iterator.getSeq2Position();
+            switch (Mutation.getRawTypeCode(mut)) {
+                case RAW_MUTATION_TYPE_SUBSTITUTION:
+                    score += scoring.getScore(Mutation.getFrom(mut), Mutation.getTo(mut)) * seq2Quality.value(seq2Pos);
+                    break;
+
+                case RAW_MUTATION_TYPE_DELETION:
+
+                    int weight = 0;
+                    int n = 0;
+                    if (seq2Pos - 1 >= seq2Range.getFrom()) {
+                        weight += seq2Quality.value(seq2Pos - 1);
+                        ++n;
+                    }
+                    if (seq2Pos < seq2Range.getTo()) {
+                        weight += seq2Quality.value(seq2Pos);
+                        ++n;
+                    }
+                    if (n == 0) // extreme case of empty input sequence
+                        weight = SequenceQuality.GOOD_QUALITY_VALUE;
+                    else
+                        weight /= n;
+                    score += scoring.getGapPenalty() * weight;
+                    break;
+
+                case RAW_MUTATION_TYPE_INSERTION:
+                    score += scoring.getGapPenalty() * seq2Quality.value(seq2Pos);
+                    break;
+
+                default:
+                    byte c = seq1.codeAt(iterator.getSeq1Position());
+                    score += scoring.getScore(c, c) * seq2Quality.value(seq2Pos);
+                    break;
+            }
+        }
+
+        assert iterator.getSeq2Position() == seq2Range.getTo();
+
+        return score;
+    }
+
     /**
      * Calculates score of alignment
      *
