@@ -20,6 +20,7 @@ import com.milaboratory.core.mutations.MutationsBuilder;
 import com.milaboratory.core.sequence.NucleotideSequence;
 
 import static java.lang.Math.max;
+import static java.lang.Math.min;
 
 /**
  * @author Dmitry Bolotin
@@ -329,13 +330,16 @@ public class BandedAffineAligner {
 
     public static BandedSemiLocalResult semiGlobalRight0(final AffineGapAlignmentScoring<NucleotideSequence> scoring,
                                                          final NucleotideSequence seq1, final NucleotideSequence seq2,
-                                                         final int offset1, final int length1, final int addedNucleotides1,
-                                                         final int offset2, final int length2, final int addedNucleotides2,
+                                                         final int offset1, final int length1, int addedNucleotides1,
+                                                         final int offset2, final int length2, int addedNucleotides2,
                                                          final int width, final MutationsBuilder<NucleotideSequence> mutations,
                                                          final MatrixCache cache) {
         int size1 = length1 + 1,
                 size2 = length2 + 1;
 
+        addedNucleotides1 = min(addedNucleotides1, length1);
+        addedNucleotides2 = min(addedNucleotides2, length2);
+
         cache.prepareMatrices(size1, size2, width, scoring);
 
         BandedMatrix main = cache.main;
@@ -430,13 +434,16 @@ public class BandedAffineAligner {
 
     public static BandedSemiLocalResult semiGlobalLeft0(final AffineGapAlignmentScoring<NucleotideSequence> scoring,
                                                         final NucleotideSequence seq1, final NucleotideSequence seq2,
-                                                        final int offset1, final int length1, final int addedNucleotides1,
-                                                        final int offset2, final int length2, final int addedNucleotides2,
+                                                        final int offset1, final int length1, int addedNucleotides1,
+                                                        final int offset2, final int length2, int addedNucleotides2,
                                                         final int width, final MutationsBuilder<NucleotideSequence> mutations,
                                                         final MatrixCache cache) {
         int size1 = length1 + 1,
                 size2 = length2 + 1;
 
+        addedNucleotides1 = min(addedNucleotides1, length1);
+        addedNucleotides2 = min(addedNucleotides2, length2);
+
         cache.prepareMatrices(size1, size2, width, scoring);
 
         BandedMatrix main = cache.main;
