@@ -1,5 +1,5 @@
 /*
- * Copyright 2015 MiLaboratory.com
+ * Copyright (c) 2022 MiLaboratories Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -32,6 +32,10 @@ import static com.milaboratory.core.mutations.Mutation.RAW_MUTATION_TYPE_DELETIO
 import static com.milaboratory.core.mutations.Mutation.RAW_MUTATION_TYPE_SUBSTITUTION;
 
 public class MultiAlignmentHelper {
+    // subject / queries nomenclature seems to be swapped here...
+    // subject here corresponds to sequence2 from alignments (so, the query sequence)
+    // queries here corresponds to sequence1 from alignments (so, the reference sequence)
+
     int minimalPositionWidth = 0;
     final String subject;
     final String[] queries;
@@ -47,12 +51,6 @@ public class MultiAlignmentHelper {
     String subjectRightTitle;
     final String[] queryRightTitles;
 
-    private MultiAlignmentHelper(String subject, String[] queries, int[] subjectPositions, int[][] queryPositions,
-                                 BitArray[] match) {
-        this(subject, queries, subjectPositions, queryPositions, match, "", new String[queries.length],
-                "", new String[queries.length]);
-    }
-
     private MultiAlignmentHelper(String subject, String[] queries, int[] subjectPositions,
                                  int[][] queryPositions, BitArray[] match,
                                  String subjectLeftTitle, String[] queryLeftTitles,
@@ -68,6 +66,32 @@ public class MultiAlignmentHelper {
         this.queryRightTitles = queryRightTitles;
     }
 
+    private MultiAlignmentHelper(String subject, String[] queries, int[] subjectPositions, int[][] queryPositions,
+                                 BitArray[] match) {
+        this(subject, queries, subjectPositions, queryPositions, match, "", new String[queries.length],
+                "", new String[queries.length]);
+    }
+
+    public String getSubject() {
+        return subject;
+    }
+
+    public String getQuery(int idx) {
+        return queries[idx];
+    }
+
+    public int[] getSubjectPositions() {
+        return subjectPositions;
+    }
+
+    public int[][] getQueryPositions() {
+        return queryPositions;
+    }
+
+    public BitArray[] getMatch() {
+        return match;
+    }
+
     public String getSubjectLeftTitle() {
         return subjectLeftTitle;
     }
@@ -308,14 +332,15 @@ public class MultiAlignmentHelper {
         public final boolean markMatchWithSpecialLetter;
         public final boolean lowerCaseMatch;
         public final boolean lowerCaseMismatch;
-        public final char matchChar;
+        public final char specialMatchChar;
         public final char outOfRangeChar;
 
-        public Settings(boolean markMatchWithSpecialLetter, boolean lowerCaseMatch, boolean lowerCaseMismatch, char matchChar, char outOfRangeChar) {
+        public Settings(boolean markMatchWithSpecialLetter, boolean lowerCaseMatch, boolean lowerCaseMismatch,
+                        char specialMatchChar, char outOfRangeChar) {
             this.markMatchWithSpecialLetter = markMatchWithSpecialLetter;
             this.lowerCaseMatch = lowerCaseMatch;
             this.lowerCaseMismatch = lowerCaseMismatch;
-            this.matchChar = matchChar;
+            this.specialMatchChar = specialMatchChar;
             this.outOfRangeChar = outOfRangeChar;
         }
     }
@@ -499,7 +524,7 @@ public class MultiAlignmentHelper {
                         mutationPointers[i]++;
                     } else {
                         if (settings.markMatchWithSpecialLetter)
-                            queryStrings[i].append(settings.matchChar);
+                            queryStrings[i].append(settings.specialMatchChar);
                         else
                             queryStrings[i].append(settings.lowerCaseMatch ? Character.toLowerCase(subjectSymbol) :
                                     subjectSymbol);
@@ -537,4 +562,4 @@ public class MultiAlignmentHelper {
         Arrays.fill(c, ' ');
         return String.valueOf(c);
     }
-}
\ No newline at end of file
+}
