@@ -1,5 +1,5 @@
 /*
- * Copyright 2015 MiLaboratory.com
+ * Copyright (c) 2022 MiLaboratories Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -43,7 +43,7 @@ public final class KAlignerParameters implements Cloneable, java.io.Serializable
                     HashMap<String, KAlignerParameters>
                     >() {
             };
-            map = GlobalObjectMappers.ONE_LINE.readValue(is, typeRef);
+            map = GlobalObjectMappers.getOneLine().readValue(is, typeRef);
         } catch (IOException ioe) {
             System.out.println("ERROR!");
             ioe.printStackTrace();
@@ -627,7 +627,7 @@ public final class KAlignerParameters implements Cloneable, java.io.Serializable
     @Override
     public String toString() {
         try {
-            return "KAlignerParameters" + GlobalObjectMappers.PRETTY.writeValueAsString(this);
+            return "KAlignerParameters" + GlobalObjectMappers.getPretty().writeValueAsString(this);
         } catch (JsonProcessingException e) {
             return "Error...";
         }
