@@ -175,7 +175,7 @@ public final class KMapper implements java.io.Serializable {
                    int offsetMask,
                    int kValue,
                    int[][] base, int[] lengths,
-                   float absoluteMinScore,  float relativeMinScore,
+                   float absoluteMinScore, float relativeMinScore,
                    float matchScore, float mismatchPenalty, float offsetShiftPenalty,
                    int minAlignmentLength, int maxIndels,
                    boolean floatingLeftBound, boolean floatingRightBound,
@@ -456,15 +456,18 @@ public final class KMapper implements java.io.Serializable {
 
         IntArrayList seedPositions = new IntArrayList((to - from) / minDistance + 2);
         int seedPosition = from;
-        seedPositions.add(seedPosition);
+        if (!sequence.containsWildcards(seedPosition, seedPosition + kValue))
+            seedPositions.add(seedPosition);
 
         Well19937c random = RandomUtil.getThreadLocalRandom();
 
         while ((seedPosition += random.nextInt(maxDistance + 1 - minDistance) + minDistance) < to - kValue)
-            seedPositions.add(seedPosition);
+            if (!sequence.containsWildcards(seedPosition, seedPosition + kValue))
+                seedPositions.add(seedPosition);
 
         //if (seedPositions.get(seedPositions.size() - 1) != to - kValue)
-        seedPositions.add(to - kValue);
+        if (!sequence.containsWildcards(to - kValue, to))
+            seedPositions.add(to - kValue);
 
         int[] seeds = new int[seedPositions.size()];
 
