@@ -1,3 +1,18 @@
+/*
+ * Copyright (c) 2022 MiLaboratories Inc.
+ *
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ *
+ *     http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
 package com.milaboratory.core.alignment.kaligner2;
 
 import com.fasterxml.jackson.core.JsonProcessingException;
@@ -694,7 +709,7 @@ public final class KAlignerParameters2 implements AbstractKAlignerParameters, Cl
     @Override
     public String toString() {
         try {
-            return "KAlignerParameters" + GlobalObjectMappers.PRETTY.writeValueAsString(this);
+            return "KAlignerParameters" + GlobalObjectMappers.getPretty().writeValueAsString(this);
         } catch (JsonProcessingException e) {
             return "Error...";
         }
