@@ -404,8 +404,8 @@ public final class KMapper2 implements java.io.Serializable {
 
                 //Detecting homopolymeric kMers and dropping them
                 //TODO:::!!!!
-//                if (((kmer ^ (kmer >>> 2)) & tMask) == 0 && ((kmer ^ (kmer << 2)) & (tMask << 2)) == 0)
-//                    continue;
+                //                if (((kmer ^ (kmer >>> 2)) & tMask) == 0 && ((kmer ^ (kmer << 2)) & (tMask << 2)) == 0)
+                //                    continue;
 
                 addKmer(holesMask, kmer, id, i);
             }
@@ -509,15 +509,18 @@ public final class KMapper2 implements java.io.Serializable {
         int seedPosition = from;
 
         // Adding firs possible position
-        seedPositions.add(seedPosition);
+        if (!sequence.containsWildcards(seedPosition, seedPosition + nValue))
+            seedPositions.add(seedPosition);
 
         // Generating random positions of seeds
         RandomGenerator random = RandomUtil.getThreadLocalRandom();
         while ((seedPosition += random.nextInt(maxDistance + 1 - minDistance) + minDistance) < to - nValue)
-            seedPositions.add(seedPosition);
+            if (!sequence.containsWildcards(seedPosition, seedPosition + nValue))
+                seedPositions.add(seedPosition);
 
         // Adding last possible position to the lis of seed positions
-        seedPositions.add(to - nValue);
+        if (!sequence.containsWildcards(to - nValue, to))
+            seedPositions.add(to - nValue);
 
         int kmer;
         final IntArrayList[] candidates = cache.candidates;
