@@ -23,7 +23,7 @@ import com.milaboratory.primitivio.PrimitivI;
 import com.milaboratory.primitivio.PrimitivO;
 import com.milaboratory.primitivio.Serializer;
 
-public class AlignmentSerializer implements Serializer<Alignment> {
+public final class AlignmentSerializer implements Serializer<Alignment> {
     @Override
     public void write(PrimitivO output, Alignment object) {
         output.writeObject(object.getSequence1());
