@@ -15,6 +15,8 @@
  */
 package com.milaboratory.core.io.sequence;
 
+import com.milaboratory.core.sequence.NSQTuple;
+import com.milaboratory.core.sequence.NSequenceWithQuality;
 import com.milaboratory.core.sequence.quality.FunctionWithIndex;
 import com.milaboratory.primitivio.annotations.Serializable;
 
@@ -33,6 +35,13 @@ public interface SequenceRead extends Iterable<SingleRead> {
 
     long getId();
 
+    default NSQTuple toTuple() {
+        NSequenceWithQuality[] data = new NSequenceWithQuality[numberOfReads()];
+        for (int i = 0; i < data.length; i++)
+            data[i] = getRead(i).getData();
+        return new NSQTuple(getId(), data);
+    }
+
     default SequenceRead mapReads(Function<SingleRead, SingleRead> mapping) {
         return SequenceReadUtil.construct(IntStream.range(0, numberOfReads())
                 .mapToObj(this::getRead)
