@@ -1,5 +1,5 @@
 /*
- * Copyright 2016 MiLaboratory.com
+ * Copyright (c) 2022 MiLaboratories Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -484,7 +484,6 @@ public final class RandomAccessFastaIndex {
          * -1 = builder finished it's work and invalidated
          */
         long currentStreamPosition = 0;
-        long lastNonLineBreakPosition = -1;
         /**
          * Counter of sequence position
          */
@@ -539,8 +538,6 @@ public final class RandomAccessFastaIndex {
 
                 // Processing line breaks
                 if (b == '\n' || b == '\r') {
-                    if (!onLineStart)
-                        lastNonLineBreakPosition = streamPosition - 1;
                     onLineStart = true;
                     continue;
                 }
@@ -564,6 +561,8 @@ public final class RandomAccessFastaIndex {
                 }
 
                 if (headerBufferPointer == -1) {
+                    if (Character.isSpaceChar(b))
+                        continue;
                     long sequencePosition = currentSequencePosition++;
                     if (sequencePosition != 0 && sequencePosition % builder.indexStep == 0)
                         builder.addIndexPoint(streamPosition);
