@@ -26,6 +26,8 @@ import java.io.File;
 import java.io.FileInputStream;
 import java.io.IOException;
 import java.io.InputStream;
+import java.nio.file.Files;
+import java.nio.file.Paths;
 
 /**
  * @author Dmitry Bolotin
@@ -62,7 +64,7 @@ public final class SingleFastqReader implements SingleReader, CanReportProgress,
      * @throws IOException in case there is problem with reading from files
      */
     public SingleFastqReader(String file, boolean replaceWildcards, boolean lazyReads) throws IOException {
-        this(new FileInputStream(file), DEFAULT_QUALITY_FORMAT, CompressionType.detectCompressionType(file),
+        this(Files.newInputStream(Paths.get(file)), DEFAULT_QUALITY_FORMAT, CompressionType.detectCompressionType(file),
                 true, DEFAULT_BUFFER_SIZE, replaceWildcards, lazyReads);
     }
 
