@@ -17,22 +17,17 @@ package com.milaboratory.core.merger;
 
 import cc.redberry.pipe.Processor;
 import com.milaboratory.core.PairedEndReadsLayout;
-import com.milaboratory.core.io.sequence.PairedRead;
 import com.milaboratory.core.merger.MergerParameters.IdentityType;
 import com.milaboratory.core.motif.BitapMatcher;
 import com.milaboratory.core.motif.BitapPattern;
 import com.milaboratory.core.motif.Motif;
 import com.milaboratory.core.motif.MotifUtils;
-import com.milaboratory.core.sequence.NSequenceWithQuality;
-import com.milaboratory.core.sequence.NucleotideSequence;
-import com.milaboratory.core.sequence.SequenceBuilder;
-import com.milaboratory.core.sequence.SequenceQualityBuilder;
+import com.milaboratory.core.sequence.*;
 
 import static com.milaboratory.core.sequence.SequencesUtils.mismatchCount;
 import static java.lang.Math.*;
 
-public final class MismatchOnlyPairedReadMerger implements Processor<PairedRead, PairedReadMergingResult>,
-                                                           java.io.Serializable {
+public final class MismatchOnlyPairedReadMerger implements Processor<NSQTuple, PairedReadMergingResult> {
     public static final int MIN_SCORE_VALUE = 0;
     final int minOverlap;
     final double minimalIdentity;
@@ -76,7 +71,7 @@ public final class MismatchOnlyPairedReadMerger implements Processor<PairedRead,
      * Creates paired-end reads merger.
      *
      * @param minOverlap              minimal number of nucleotide in overlap region
-     * @param minimalIdentity         maximal allowed percent of mismatches in overlap region
+     * @param minimalIdentity         minimal allowed percent of matched letters in overlap region
      * @param identityType            identity type
      * @param maxScoreValue           maximal output quality score value
      * @param qualityMergingAlgorithm algorithm to infer quality of merged reads from it's pairs
@@ -107,11 +102,11 @@ public final class MismatchOnlyPairedReadMerger implements Processor<PairedRead,
     }
 
     public PairedReadMergingResult merge(NSequenceWithQuality read1p, NSequenceWithQuality read2p,
-                                         PairedRead pairedRead) {
+                                         NSQTuple input) {
         // If there is no sufficient letters in one of read overlapping is impossible
         if (read1p.size() < minOverlap || read2p.size() < minOverlap)
             // Return failed result
-            return new PairedReadMergingResult(pairedRead);
+            return new PairedReadMergingResult(input);
 
         PairedReadMergingResult ret = null;
 
@@ -161,7 +156,7 @@ public final class MismatchOnlyPairedReadMerger implements Processor<PairedRead,
                         read2, 0,
                         overlap);
                 if (identity >= minimalIdentity) {
-                    tmp = new PairedReadMergingResult(pairedRead, overlap(read1, read2, matchPosition),
+                    tmp = new PairedReadMergingResult(input, overlap(read1, read2, matchPosition),
                             overlap, mismatches, strand, swapped ? -matchPosition : matchPosition, identityType, identity);
                     break;
                 }
@@ -180,7 +175,7 @@ public final class MismatchOnlyPairedReadMerger implements Processor<PairedRead,
                         overlap);
                 if (identity >= minimalIdentity) {
                     final int offset = min(matchPosition - read2.size(), 0);
-                    tmp = new PairedReadMergingResult(pairedRead, overlap(read1, read2, offset),
+                    tmp = new PairedReadMergingResult(input, overlap(read1, read2, offset),
                             overlap, mismatches, strand, swapped ? -offset : offset, identityType, identity);
                     break;
                 }
@@ -190,7 +185,7 @@ public final class MismatchOnlyPairedReadMerger implements Processor<PairedRead,
         }
 
         if (ret == null)
-            return new PairedReadMergingResult(pairedRead);
+            return new PairedReadMergingResult(input);
         else
             return ret;
     }
@@ -231,9 +226,11 @@ public final class MismatchOnlyPairedReadMerger implements Processor<PairedRead,
     }
 
     @Override
-    public PairedReadMergingResult process(PairedRead pairedRead) {
-        NSequenceWithQuality read1p = pairedRead.getR1().getData();
-        NSequenceWithQuality read2p = pairedRead.getR2().getData();
+    public PairedReadMergingResult process(NSQTuple pairedRead) {
+        if (pairedRead.size() != 2)
+            throw new IllegalArgumentException("Illegal number of elements for merging.");
+        NSequenceWithQuality read1p = pairedRead.get(0);
+        NSequenceWithQuality read2p = pairedRead.get(1);
         return merge(read1p, read2p, pairedRead);
     }
 
