@@ -15,15 +15,15 @@
  */
 package com.milaboratory.core.merger;
 
-import com.milaboratory.core.io.sequence.PairedRead;
 import com.milaboratory.core.merger.MergerParameters.IdentityType;
+import com.milaboratory.core.sequence.NSQTuple;
 import com.milaboratory.core.sequence.NSequenceWithQuality;
 
 public class PairedReadMergingResult implements java.io.Serializable {
     public static final int MATCH_SCORE = 2;
     public static final int MISMATCH_SCORE = -5;
 
-    final PairedRead originalRead;
+    final NSQTuple originalSequences;
     final NSequenceWithQuality overlappedSequence;
     final int overlap;
     final int errors;
@@ -37,8 +37,8 @@ public class PairedReadMergingResult implements java.io.Serializable {
      *
      * @param originalRead original read
      */
-    public PairedReadMergingResult(PairedRead originalRead) {
-        this.originalRead = originalRead;
+    public PairedReadMergingResult(NSQTuple originalSequences) {
+        this.originalSequences = originalSequences;
         this.overlappedSequence = null;
         this.overlap = 0;
         this.errors = -1;
@@ -56,10 +56,10 @@ public class PairedReadMergingResult implements java.io.Serializable {
      * @param overlap            number of overlapped nucleotides
      * @param errors             number of mismatches/insertions/deletions found in overlapping region
      */
-    public PairedReadMergingResult(PairedRead originalRead, NSequenceWithQuality overlappedSequence,
+    public PairedReadMergingResult(NSQTuple originalSequences, NSequenceWithQuality overlappedSequence,
                                    int overlap, int errors, boolean reversed, int offset,
                                    IdentityType identityType, double identity) {
-        this.originalRead = originalRead;
+        this.originalSequences = originalSequences;
         this.overlappedSequence = overlappedSequence;
         this.overlap = overlap;
         this.errors = errors;
@@ -73,8 +73,8 @@ public class PairedReadMergingResult implements java.io.Serializable {
         return overlappedSequence != null;
     }
 
-    public PairedRead getOriginalRead() {
-        return originalRead;
+    public NSQTuple getOriginalSequences() {
+        return originalSequences;
     }
 
     public NSequenceWithQuality getOverlappedSequence() {
@@ -118,14 +118,14 @@ public class PairedReadMergingResult implements java.io.Serializable {
 
         if (overlap != that.overlap) return false;
         if (errors != that.errors) return false;
-        if (!originalRead.equals(that.originalRead)) return false;
+        if (!originalSequences.equals(that.originalSequences)) return false;
         return overlappedSequence.equals(that.overlappedSequence);
 
     }
 
     @Override
     public int hashCode() {
-        int result = originalRead.hashCode();
+        int result = originalSequences.hashCode();
         result = 31 * result + overlappedSequence.hashCode();
         result = 31 * result + overlap;
         result = 31 * result + errors;
@@ -135,7 +135,7 @@ public class PairedReadMergingResult implements java.io.Serializable {
     @Override
     public String toString() {
         return "PairedReadMergingResult{" +
-                "originalRead=" + originalRead +
+                "originalSequences=" + originalSequences +
                 ", overlappedSequence=" + overlappedSequence +
                 ", overlap=" + overlap +
                 ", errors=" + errors +
