@@ -1,5 +1,5 @@
 /*
- * Copyright 2015 MiLaboratory.com
+ * Copyright (c) 2022 MiLaboratories Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,7 +15,7 @@
  */
 package com.milaboratory.core.motif;
 
-public interface BitapMatcher extends java.io.Serializable {
+public interface BitapMatcher {
     int findNext();
 
     int getNumberOfErrors();
