@@ -22,7 +22,7 @@ import com.milaboratory.util.BitArray;
  * Use {@link Motif#getBitapPattern()} to create bitap pattern.
  */
 public final class BitapPattern implements java.io.Serializable {
-    final Motif motif;
+    final Motif<?> motif;
     final BitapData
             mainData,
             secondaryData;
@@ -36,7 +36,7 @@ public final class BitapPattern implements java.io.Serializable {
     /**
      * Use {@link Motif#getBitapPattern()} to create bitap pattern.
      */
-    BitapPattern(Motif motif, BitapData mainData, BitArray exactMask) {
+    BitapPattern(Motif<?> motif, BitapData mainData, BitArray exactMask) {
         this.motif = motif;
         this.mainData = mainData;
         this.exactMask = exactMask;
@@ -46,7 +46,7 @@ public final class BitapPattern implements java.io.Serializable {
         } else
             this.secondaryData = mainData.toSecondary(exactMask);
 
-        MotifWithExactMask me = new MotifWithExactMask(motif, exactMask);
+        MotifWithExactMask<?> me = new MotifWithExactMask(motif, exactMask);
         this.matchScore = me.defaultMatchScore;
         this.mismatchScore = me.mismatchScore;
         this.averageMismatchPenalty = me.averageMismatchPenalty;
