@@ -22,7 +22,7 @@ import com.milaboratory.core.sequence.Sequence;
 import com.milaboratory.core.sequence.Wildcard;
 import com.milaboratory.util.BitArray;
 
-import java.util.Arrays;
+import static com.milaboratory.core.motif.LongArrayBitHelper.*;
 
 /**
  * Data structure for efficient exact and fuzzy matching/searching of sequences (wildcard-aware).
@@ -120,25 +120,20 @@ public final class Motif<S extends Sequence<S>> implements java.io.Serializable
     }
 
     public BitapPattern getBitapPattern() {
-        if (size >= 64)
-            throw new RuntimeException("Supports motifs with length less then 64.");
         return bitapPattern;
     }
 
     public BitapPattern toBitapPattern(BitArray exactMask) {
-        if (size >= 64)
-            return null;
         int aSize = alphabet.size();
-        long[] patternMask = new long[aSize],
-                reversePatternMask = new long[aSize];
-        Arrays.fill(patternMask, ~0);
-        Arrays.fill(reversePatternMask, ~0);
-        int p = 0;
-        for (int i = 0; i < aSize; ++i)
-            for (int j = 0; j < size; ++j)
-                if (data.get(p++)) {
-                    patternMask[i] &= ~(1L << j);
-                    reversePatternMask[i] &= ~(1L << (size - j - 1));
+        int segments = upDivideBy64(size);
+        long[][] patternMask = ones2D(aSize, segments),
+                reversePatternMask = ones2D(aSize, segments);
+        int i = 0;
+        for (int l = 0; l < aSize; ++l)
+            for (int p = 0; p < size; ++p)
+                if (data.get(i++)) {
+                    resetBitLittleEndian(patternMask[l],  p);
+                    resetBitLittleEndian(reversePatternMask[l], size - p - 1);
                 }
         BitapData mainData = new BitapData(size, patternMask, reversePatternMask);
         return new BitapPattern(this, mainData, exactMask);
