@@ -1,5 +1,5 @@
 /*
- * Copyright 2015 MiLaboratory.com
+ * Copyright (c) 2022 MiLaboratories Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -17,6 +17,8 @@ package com.milaboratory.core.mutations;
 
 import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
 import com.fasterxml.jackson.databind.annotation.JsonSerialize;
+import com.google.common.hash.Hasher;
+import com.google.common.hash.Hashing;
 import com.milaboratory.core.Range;
 import com.milaboratory.core.sequence.*;
 import com.milaboratory.primitivio.annotations.Serializable;
@@ -104,6 +106,18 @@ public final class Mutations<S extends Sequence<S>>
         return getRawTypeCode(mutations[index]);
     }
 
+    public boolean isSubstitutionByIndex(int index) {
+        return isSubstitution(mutations[index]);
+    }
+
+    public boolean isDeletionByIndex(int index) {
+        return isDeletion(mutations[index]);
+    }
+
+    public boolean isInsertionByIndex(int index) {
+        return isInsertion(mutations[index]);
+    }
+
     public MutationType getTypeByIndex(int index) {
         return getType(mutations[index]);
     }
@@ -261,6 +275,30 @@ public final class Mutations<S extends Sequence<S>>
         return delta;
     }
 
+    /**
+     * Returns maximal and minimal shift due to indels created by the mutations object.
+     *
+     * @return maximal and minimal shift due to indels
+     */
+    public Range getShiftRange() {
+        int delta = 0, from = 0, to = 0;
+
+        for (int mut : mutations) {
+            switch (mut & MUTATION_TYPE_MASK) {
+                case RAW_MUTATION_TYPE_DELETION:
+                    --delta;
+                    break;
+                case RAW_MUTATION_TYPE_INSERTION:
+                    ++delta;
+                    break;
+            }
+            from = Math.min(from, delta);
+            to = Math.max(to, delta);
+        }
+
+        return new Range(from, to);
+    }
+
     /**
      * Concatenates this and other
      */
@@ -734,11 +772,15 @@ public final class Mutations<S extends Sequence<S>>
         return Arrays.equals(mutations, mutations1.mutations);
     }
 
+    @SuppressWarnings("UnstableApiUsage")
     @Override
     public int hashCode() {
-        int result = alphabet.hashCode();
-        result = 31 * result + Arrays.hashCode(mutations);
-        return result;
+        Hasher hasher = Hashing.murmur3_32_fixed()
+                .newHasher(4 + mutations.length)
+                .putInt(getAlphabet().hashCode());
+        for (int mutation : mutations)
+            hasher.putInt(mutation);
+        return hasher.hash().hashCode();
     }
 
     /**
