@@ -1,5 +1,5 @@
 /*
- * Copyright 2015 MiLaboratory.com
+ * Copyright (c) 2022 MiLaboratories Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -376,6 +376,9 @@ public final class MutationsUtil {
     public static Mutations<AminoAcidSequence> nt2aa(NucleotideSequence seq1, Mutations<NucleotideSequence> mutations,
                                                      TranslationParameters translationParameters,
                                                      int maxShiftedTriplets) {
+        if (mutations.isEmpty())
+            return Mutations.EMPTY_AMINO_ACID_MUTATIONS;
+
         AminoAcidSequence aaSeq1 = AminoAcidSequence.translate(seq1, translationParameters);
         NucleotideSequence seq2 = mutations.mutate(seq1);
         AminoAcidSequence aaSeq2 = AminoAcidSequence.translate(seq2, translationParameters);
@@ -384,14 +387,16 @@ public final class MutationsUtil {
         int aaP2, prevAAP2 = -1;
         int ntP1, ntP2;
         int shiftedTriplets = 0;
+
         for (int aaP1 = 0; aaP1 <= aaSeq1.size(); aaP1++) {
             if (aaP1 != aaSeq1.size()) {
                 ntP1 = AminoAcidSequence.convertAAPositionToNt(aaP1, seq1.size(), translationParameters);
                 ntP2 = mutations.convertToSeq2Position(ntP1);
+
                 if (ntP2 < 0)
                     ntP2 = -ntP2 - 2;
 
-                if (ntP2 < 0)
+                if (ntP2 == -1)
                     aaP2 = -1;
                 else {
                     AminoAcidSequencePosition pos = AminoAcidSequence.convertNtPositionToAA(ntP2, seq2.size(), translationParameters);
@@ -458,10 +463,8 @@ public final class MutationsUtil {
         int[] mapping = new int[mutations.size()];
         Arrays.fill(mapping, -1);
 
-        // Already calculates in nt2aa()
         AminoAcidSequence aaSeq1 = AminoAcidSequence.translate(seq1, translationParameters);
-        NucleotideSequence seq2 = mutations.mutate(seq1);
-        AminoAcidSequence aaSeq2 = AminoAcidSequence.translate(seq2, translationParameters);
+        int seq2Size = seq1.size() + mutations.getLengthDelta();
 
         AlignmentIteratorForward<NucleotideSequence> ntIterator = new AlignmentIteratorForward<>(mutations,
                 new Range(0, seq1.size()));
@@ -477,7 +480,7 @@ public final class MutationsUtil {
             AminoAcidSequencePosition aaSeq1Position = AminoAcidSequence.convertNtPositionToAA(
                     ntIterator.getSeq1Position(), seq1.size(), translationParameters);
             AminoAcidSequencePosition aaSeq2Position = AminoAcidSequence.convertNtPositionToAA(
-                    ntIterator.getSeq2Position(), seq2.size(), translationParameters);
+                    ntIterator.getSeq2Position(), seq2Size, translationParameters);
 
             if (aaSeq1Position == null || aaSeq2Position == null)
                 continue;
