@@ -26,12 +26,41 @@ import com.milaboratory.primitivio.PrimitivI;
 import com.milaboratory.primitivio.PrimitivO;
 import com.milaboratory.primitivio.Serializer;
 
+import java.io.ByteArrayInputStream;
 import java.io.IOException;
 
 final class IO {
     private IO() {
     }
 
+    public static class ShortSequenceSetSerializer implements Serializer<ShortSequenceSet> {
+        @Override
+        public void write(PrimitivO output, ShortSequenceSet obj) {
+            byte[] bytes = obj.serialize();
+            output.writeObject(bytes);
+        }
+
+        @Override
+        public ShortSequenceSet read(PrimitivI input) {
+            byte[] bytes = input.readObject(byte[].class);
+            try {
+                return ShortSequenceSet.deserialize(new ByteArrayInputStream(bytes));
+            } catch (IOException e) {
+                throw new RuntimeException(e);
+            }
+        }
+
+        @Override
+        public boolean isReference() {
+            return true;
+        }
+
+        @Override
+        public boolean handlesReference() {
+            return false;
+        }
+    }
+
     public static class AlphabetSerializer implements Serializer<Alphabet> {
         @Override
         public void write(PrimitivO output, Alphabet object) {
