@@ -62,7 +62,7 @@ public class NSequenceWithQualityBuilder implements SeqBuilder<NSequenceWithQual
     }
 
     public NSequenceWithQualityBuilder append(char symbol, byte quality) {
-        return append(NucleotideSequence.ALPHABET.symbolToCode(symbol), quality);
+        return append(NucleotideSequence.ALPHABET.symbolToCodeWithException(symbol), quality);
     }
 
     @Override
