@@ -1,5 +1,5 @@
 /*
- * Copyright 2015 MiLaboratory.com
+ * Copyright (c) 2022 MiLaboratories Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -16,12 +16,15 @@
 package com.milaboratory.core.sequence;
 
 import com.milaboratory.core.Range;
+import com.milaboratory.core.mutations.Mutation;
 import com.milaboratory.util.Bit2Array;
 import com.milaboratory.util.HashFunctions;
 
 import java.util.HashSet;
 import java.util.Set;
 
+import static com.milaboratory.core.mutations.Mutation.*;
+
 /**
  * Utility methods for sequences.
  *
@@ -182,4 +185,19 @@ public final class SequencesUtils {
         }
         return longest;
     }
+
+    /** Quality score for the mutated position in the sequence */
+    public static byte qualityForMutation(SequenceQuality qual, int mutation) {
+        int p = getPosition(mutation);
+        if (isDeletion(mutation) || isSubstitution(mutation))
+            return qual.value(p);
+        else {
+            if (p == 0)
+                return qual.value(0);
+            else if (p == qual.size())
+                return qual.value(qual.size() - 1);
+            else
+                return qual.minValue(p - 1, p + 1);
+        }
+    }
 }
\ No newline at end of file
