@@ -1,5 +1,5 @@
 /*
- * Copyright 2016 MiLaboratory.com
+ * Copyright (c) 2022 MiLaboratories Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -17,7 +17,7 @@ package com.milaboratory.core.sequence.quality;
 
 import com.milaboratory.core.sequence.SequenceQuality;
 
-public class MaximalQualityAggregator implements QualityAggregator {
+public final class MaximalQualityAggregator implements QualityAggregator {
     final byte[] data;
 
     public MaximalQualityAggregator(int size) {
