@@ -17,7 +17,8 @@ package com.milaboratory.core.sequence.quality;
 
 import com.milaboratory.core.Range;
 import com.milaboratory.core.io.sequence.SequenceRead;
+import com.milaboratory.core.sequence.NSQTuple;
 
 public interface ReadTrimmerListener {
-    void onSequence(SequenceRead originalRead, int readIndex, Range range, boolean trimmed);
+    void onSequence(NSQTuple original, int readIndex, Range range, boolean trimmed);
 }
