@@ -19,9 +19,10 @@ import cc.redberry.pipe.Processor;
 import cc.redberry.primitives.Filter;
 import com.milaboratory.core.Range;
 import com.milaboratory.core.io.sequence.SequenceRead;
+import com.milaboratory.core.sequence.NSQTuple;
 import com.milaboratory.core.sequence.NSequenceWithQuality;
 
-public final class ReadTrimmerProcessor<R extends SequenceRead> implements Processor<R, R> {
+public final class ReadTrimmerProcessor implements Processor<NSQTuple, NSQTuple> {
     public static final Filter<SequenceRead> NON_EMPTY_READ_FILTER = object -> {
         for (int i = 0; i < object.numberOfReads(); i++) {
             if (object.getRead(i).getData().size() != 0)
@@ -43,22 +44,20 @@ public final class ReadTrimmerProcessor<R extends SequenceRead> implements Proce
     }
 
     @Override
-    public R process(R input) {
+    public NSQTuple process(NSQTuple input) {
         //noinspection unchecked
-        return (R) input.mapReadsWithIndex((i, singleRead) ->
-                singleRead.mapSequence(nsq -> {
-                    Range range = QualityTrimmer.bestIsland(nsq.getQuality(), trimmingParameters);
-                    if (listener != null)
-                        listener.onSequence(input, i, range,
-                                range == null
-                                        || range.getLower() != 0
-                                        || range.getUpper() != nsq.size());
-                    if (range == null)
-                        return NSequenceWithQuality.EMPTY;
-                    if (range.getLower() == 0 && range.getUpper() == nsq.size())
-                        return nsq;
-                    return nsq.getRange(range);
-                })
-        );
+        return input.mapWithIndex((i, nsq) -> {
+            Range range = QualityTrimmer.bestIsland(nsq.getQuality(), trimmingParameters);
+            if (listener != null)
+                listener.onSequence(input, i, range,
+                        range == null
+                                || range.getLower() != 0
+                                || range.getUpper() != nsq.size());
+            if (range == null)
+                return NSequenceWithQuality.EMPTY;
+            if (range.getLower() == 0 && range.getUpper() == nsq.size())
+                return nsq;
+            return nsq.getRange(range);
+        });
     }
 }
