@@ -16,35 +16,39 @@
 package com.milaboratory.primitivio;
 
 import com.fasterxml.jackson.core.JsonProcessingException;
-import com.milaboratory.util.GlobalObjectMappers;
+import com.fasterxml.jackson.databind.ObjectMapper;
 
 import java.io.IOException;
 import java.util.Objects;
 import java.util.function.Function;
 
 public final class JSONSerializer implements Serializer {
+    final ObjectMapper objectMapper;
     final Class<?> type;
     final Function<byte[], byte[]> preprocessor;
 
     /**
      * Constructor for backward-compatibility mocking serializers
      *
+     * @param objectMapper object mapper to use during serialization and deserialization
      * @param type         target type
      * @param preprocessor JSON string content preprocessor
      */
-    public JSONSerializer(Class<?> type, Function<byte[], byte[]> preprocessor) {
+    public JSONSerializer(ObjectMapper objectMapper, Class<?> type, Function<byte[], byte[]> preprocessor) {
+        this.objectMapper = Objects.requireNonNull(objectMapper);
         this.type = Objects.requireNonNull(type);
         this.preprocessor = preprocessor;
     }
 
     /**
      * Normal constructor, not intended for direct use,
-     * use com.milaboratory.primitivio.annotations.Serializable#asJson() instead.
+     * use {@link com.milaboratory.primitivio.annotations.Serializable#asJson()} instead.
      *
-     * @param type target type
+     * @param objectMapper object mapper to use during serialization and deserialization
+     * @param type         target type
      */
-    public JSONSerializer(Class<?> type) {
-        this(type, null);
+    public JSONSerializer(ObjectMapper objectMapper, Class<?> type) {
+        this(objectMapper, type, null);
     }
 
     @Override
@@ -52,7 +56,7 @@ public final class JSONSerializer implements Serializer {
         try {
             if (preprocessor != null)
                 throw new IllegalStateException("");
-            byte[] bytes = GlobalObjectMappers.getOneLine().writeValueAsBytes(object);
+            byte[] bytes = objectMapper.writeValueAsBytes(object);
             output.writeObject(bytes);
         } catch (JsonProcessingException e) {
             throw new RuntimeException(e);
@@ -63,7 +67,7 @@ public final class JSONSerializer implements Serializer {
     public Object read(PrimitivI input) {
         byte[] bytes = input.readObject(byte[].class);
         try {
-            return GlobalObjectMappers.getOneLine().readValue(
+            return objectMapper.readValue(
                     preprocessor != null
                             ? preprocessor.apply(bytes)
                             : bytes,
