@@ -19,7 +19,6 @@ import com.milaboratory.primitivio.annotations.CustomSerializer;
 import com.milaboratory.primitivio.annotations.Serializable;
 
 import java.lang.reflect.Constructor;
-import java.lang.reflect.Type;
 import java.util.ArrayList;
 import java.util.HashMap;
 import java.util.List;
@@ -101,7 +100,8 @@ public final class SerializersManager {
         if (annotation.asJson()) {
             if (defaultSerializer != null)
                 throw new RuntimeException("'asJson' and 'by' parameters are not compatible.");
-            defaultSerializer = new JSONSerializer(type);
+            ObjectMapperProvider omp = instantiate(annotation.objectMapperBy());
+            defaultSerializer = new JSONSerializer(omp.getObjectMapper(), type);
         }
 
         CustomSerializer[] css = annotation.custom();
@@ -135,9 +135,9 @@ public final class SerializersManager {
         );
     }
 
-    static Serializer instantiate(Class<? extends Serializer> cl) {
+    static <T> T instantiate(Class<? extends T> cl) {
         boolean initialAccessibility = true;
-        Constructor<? extends Serializer> constructor = null;
+        Constructor<? extends T> constructor = null;
         try {
             constructor = cl.getConstructor();
             initialAccessibility = constructor.isAccessible();
