@@ -105,7 +105,7 @@ public final class Util {
         return list;
     }
 
-    public static <K, V> void writeMap(Map<K, V> map, PrimitivO output) {
+    public static <K, V> void writeMap(SortedMap<K, V> map, PrimitivO output) {
         output.writeVarInt(map.size());
         for (Map.Entry<K, V> entry : map.entrySet()) {
             output.writeObject(entry.getKey());
