@@ -15,6 +15,7 @@
  */
 package com.milaboratory.primitivio.annotations;
 
+import com.milaboratory.primitivio.ObjectMapperProvider;
 import com.milaboratory.primitivio.Serializer;
 
 import java.lang.annotation.ElementType;
@@ -25,9 +26,18 @@ import java.lang.annotation.Target;
 @Retention(RetentionPolicy.RUNTIME)
 @Target(ElementType.TYPE)
 public @interface Serializable {
+    /** Set serializer for this class */
     Class<? extends Serializer> by() default Serializer.class;
 
+    /** Use for polymorphic serialization */
     CustomSerializer[] custom() default {};
 
+    /**
+     * If true, Jackson will be used to covert object to string, and resulting string will be serialized, the opposite
+     * operation will be used for deserialization.
+     */
     boolean asJson() default false;
+
+    /** Allows to customise object mapper that will be used for serialization if {@link #asJson()} option is true. */
+    Class<? extends ObjectMapperProvider> objectMapperBy() default ObjectMapperProvider.Default.class;
 }
