@@ -579,7 +579,7 @@ public final class PrimitivOBlocks<O> extends PrimitivIOBlocksAbstract {
                 if (sumWritten.addAndGet(result) < limit) {
                     if (src.limit() != limit - sumWritten.get())
                         // Just in case
-                        throw new IllegalStateException("Unexpected buffer behaviour");
+                        throw new IllegalStateException("Unexpected buffer behaviour. This might be a sign that you ran out of storage.");
 
                     // Recursive write operation
                     channel.write(src, attachment, this);
