@@ -25,8 +25,10 @@ import com.milaboratory.primitivio.annotations.Serializable;
 import org.apache.commons.io.IOUtils;
 
 import java.io.IOException;
+import java.math.BigInteger;
 import java.nio.ByteBuffer;
 import java.nio.channels.FileChannel;
+import java.nio.channels.ReadableByteChannel;
 import java.nio.file.Files;
 import java.nio.file.Path;
 import java.nio.file.StandardOpenOption;
@@ -34,6 +36,7 @@ import java.nio.file.attribute.BasicFileAttributes;
 import java.security.MessageDigest;
 import java.security.NoSuchAlgorithmException;
 import java.util.Arrays;
+import java.util.List;
 import java.util.Objects;
 
 /**
@@ -175,6 +178,77 @@ public class LightFileDescriptor {
         }
     }
 
+    static final int CHUNK_SIZE = 1 << 15; // 32kb
+
+    public static void addLightFileCache(Path file, MessageDigest md) throws IOException {
+        long fileSize = Files.size(file);
+
+        // Adding file size to MD5 sum
+        md.update(ByteBuffer.allocate(Long.SIZE / Byte.SIZE).putLong(fileSize).array());
+
+        // Adding actual content checksum
+        try (FileChannel channel = FileChannel.open(file, StandardOpenOption.READ)) {
+            if (fileSize < CHUNK_SIZE) {
+                ByteBuffer buff = ByteBuffer.allocate((int) fileSize);
+                read(channel, buff, md);
+            } else {
+                ByteBuffer buff = ByteBuffer.allocate(CHUNK_SIZE);
+
+                // Reading first chunk
+                channel.position(0);
+                read(channel, buff, md);
+
+                // Reading middle chunk
+                channel.position((fileSize - CHUNK_SIZE) / 2);
+                read(channel, buff, md);
+
+                // Reading last chunk
+                channel.position(fileSize - CHUNK_SIZE);
+                read(channel, buff, md);
+            }
+        }
+    }
+
+    public static BigInteger lightFileCacheAsBI(Path file) throws IOException {
+        try {
+            MessageDigest md = MessageDigest.getInstance("MD5");
+            addLightFileCache(file, md);
+            return new BigInteger(1, md.digest());
+        } catch (NoSuchAlgorithmException e) {
+            return null;
+        }
+    }
+
+    /** No dependence on file order */
+    public static byte[] calculateCommutativeLightHash(List<Path> files) {
+        try {
+            BigInteger sum = BigInteger.ZERO;
+            for (Path f : files) {
+                BigInteger hash = lightFileCacheAsBI(f);
+                if (hash == null)
+                    return null;
+                sum = sum.add(hash);
+            }
+            MessageDigest md = MessageDigest.getInstance("MD5");
+            // Adding sum (this will be a hash of a sum of hashes)
+            md.update(sum.toByteArray());
+            return md.digest();
+        } catch (NoSuchAlgorithmException | IOException e) {
+            return null;
+        }
+    }
+
+    private static void read(ReadableByteChannel input, ByteBuffer buffer, MessageDigest md) throws IOException {
+        buffer.clear();
+        while (buffer.remaining() > 0)
+            if (-1 == input.read(buffer))
+                break;
+        if (buffer.remaining() != 0)
+            throw new IllegalArgumentException();
+        buffer.flip();
+        md.update(buffer);
+    }
+
     @Override
     public boolean equals(Object o) {
         if (this == o) return true;
