@@ -50,6 +50,11 @@ public interface ObjectSerializer<O> {
          */
         private SerializersManager sm;
 
+        public PrimitivIOObjectSerializer(Class<O> clazz, SerializersManager sm) {
+            this.clazz = clazz;
+            this.sm = sm;
+        }
+
         public PrimitivIOObjectSerializer(Class<O> clazz) {
             this.clazz = clazz;
             this.sm = new SerializersManager();
