@@ -16,8 +16,11 @@
 package com.milaboratory.util;
 
 import com.fasterxml.jackson.annotation.JsonAutoDetect;
-import com.milaboratory.util.ReportHelper;
 
+/**
+ * Immutable, JSON deserializable version of this report. Returns null if json deserialization is not intended or not
+ * supported.
+ */
 @JsonAutoDetect(
         getterVisibility = JsonAutoDetect.Visibility.NONE,
         isGetterVisibility = JsonAutoDetect.Visibility.NONE,
@@ -26,5 +29,6 @@ import com.milaboratory.util.ReportHelper;
         creatorVisibility = JsonAutoDetect.Visibility.NONE
 )
 public interface Report {
+    /** Prints report to a PrintStream provided by ReportHelper */
     void writeReport(ReportHelper helper);
 }
