@@ -1,5 +1,5 @@
 /*
- * Copyright 2015 MiLaboratory.com
+ * Copyright (c) 2022 MiLaboratories Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -31,6 +31,7 @@ import java.util.Properties;
 @Serializable(asJson = true)
 public class VersionInfo {
     final String version, revision, name, branch, host;
+    final boolean production;
     final Date timestamp;
 
     public VersionInfo(@JsonProperty("version") String version,
@@ -38,12 +39,14 @@ public class VersionInfo {
                        @JsonProperty("name") String name,
                        @JsonProperty("branch") String branch,
                        @JsonProperty("host") String host,
+                       @JsonProperty("production") boolean production,
                        @JsonProperty("timestamp") Date timestamp) {
         this.version = version;
         this.revision = revision;
         this.name = name;
         this.branch = branch;
         this.host = host;
+        this.production = production;
         this.timestamp = timestamp;
     }
 
@@ -67,6 +70,10 @@ public class VersionInfo {
         return host;
     }
 
+    public boolean isProductionBuild() {
+        return production;
+    }
+
     public Date getTimestamp() {
         return timestamp;
     }
@@ -81,12 +88,13 @@ public class VersionInfo {
                 Objects.equals(name, that.name) &&
                 Objects.equals(branch, that.branch) &&
                 Objects.equals(host, that.host) &&
+                production == that.production &&
                 Objects.equals(timestamp, that.timestamp);
     }
 
     @Override
     public int hashCode() {
-        return Objects.hash(version, revision, name, branch, host, timestamp);
+        return Objects.hash(version, revision, name, branch, host, production, timestamp);
     }
 
     @Override
@@ -97,6 +105,7 @@ public class VersionInfo {
                 ", name='" + name + '\'' +
                 ", branch='" + branch + '\'' +
                 ", host='" + host + '\'' +
+                ", production=" + production +
                 ", timestamp=" + timestamp +
                 '}';
     }
@@ -125,7 +134,7 @@ public class VersionInfo {
             case "repseqio":
                 return "io.repseq.util.RepseqIOVersionInfo";
             default:
-                return resourceName;
+                return "com.milaboratory.util.VersionInfo";
         }
     }
 
@@ -141,6 +150,7 @@ public class VersionInfo {
                 properties.getProperty("name"),
                 properties.getProperty("branch"),
                 properties.getProperty("host"),
+                "true".equals(properties.getProperty("production")),
                 new Date(Long.parseLong(properties.getProperty("timestamp"))));
     }
 }
