@@ -98,6 +98,9 @@ public class HashSorter<T> {
             totalFilesUsed = new AtomicLong(),
             maxDepth = new AtomicLong();
 
+    /** Will store effective uncompressed object size after the first collation step */
+    private final AtomicLong uncompressedObjectSize = new AtomicLong(-1);
+
     /**
      * Creates hash sorter. Actual sorting starts on {@link #port(OutputPort)} invocation.
      *
@@ -160,6 +163,12 @@ public class HashSorter<T> {
         return nodeInfos.size();
     }
 
+    public long getUncopressedObjectSize() {
+        if (uncompressedObjectSize.get() == -1)
+            throw new IllegalStateException("Not yet initialized.");
+        return uncompressedObjectSize.get();
+    }
+
     public void printStat() {
         printStat(true);
     }
@@ -326,7 +335,6 @@ public class HashSorter<T> {
         }
     }
 
-
     private final class Collator implements Runnable {
         final CollatorAddress address;
         final OutputPort<T> source;
@@ -490,6 +498,9 @@ public class HashSorter<T> {
 
                 // Saving actual object size
                 this.objectSize = objectSize;
+                // Globally saving object size, this has no effect on the sorting process,
+                // used only to report the value to the user of HashSorter
+                uncompressedObjectSize.compareAndSet(-1, objectSize);
             } catch (IOException e) {
                 throw new RuntimeException(e);
             }
