@@ -17,6 +17,7 @@ package com.milaboratory.core;
 
 import com.milaboratory.core.io.sequence.PairedRead;
 import com.milaboratory.core.io.sequence.SingleReadImpl;
+import com.milaboratory.core.sequence.NSQTuple;
 import com.milaboratory.core.sequence.NSequenceWithQuality;
 import org.junit.Assert;
 import org.junit.Test;
@@ -27,10 +28,7 @@ public class PairedEndReadsLayoutTest {
         NSequenceWithQuality r1 = new NSequenceWithQuality("ATTAGACA");
         NSequenceWithQuality r2 = new NSequenceWithQuality("GATCAGT");
 
-        PairedRead pr = new PairedRead(
-                new SingleReadImpl(0, r1, "R1"),
-                new SingleReadImpl(0, r2, "R2")
-        );
+        NSQTuple pr = new NSQTuple(0, r1, r2);
 
         Target[] targets = PairedEndReadsLayout.ReverseOnly.createTargets(pr);
         Assert.assertEquals(1, targets.length);
