@@ -43,7 +43,7 @@ public class AlignmentIteratorTest {
             I0A I3G SA3G DC6 I8T I8T
          */
 
-        AlignmentIteratorTestCase case1 = new AlignmentIteratorTestCase("I0A I3G SA3G DC6 I8T I8T",
+        AlignmentIteratorTestCase case1 = new AlignmentIteratorTestCase("I0A I3G SA3G DC6 I8T I8T", 11,
                 new int[]{0, 0, 1, 2, 3, 3, 4, 5, 6, 7, 8, 8},
                 new int[]{0, 1, 2, 3, 4, 5, 6, 7, 8, 8, 9, 10},
                 new int[]{0, 1, 1, 1, 1, 2, 3, 3, 3, 4, 4, 5},
@@ -72,7 +72,7 @@ public class AlignmentIteratorTest {
             I0A I3G SA3G DC6
          */
 
-        AlignmentIteratorTestCase case2F = new AlignmentIteratorTestCase("I0A I3G SA3G DC6",
+        AlignmentIteratorTestCase case2F = new AlignmentIteratorTestCase("I0A I3G SA3G DC6", 9,
                 new int[]{0, 0, 1, 2, 3, 3, 4, 5, 6, 7},
                 new int[]{0, 1, 2, 3, 4, 5, 6, 7, 8, 8},
                 new int[]{0, 1, 1, 1, 1, 2, 3, 3, 3, 4},
@@ -99,7 +99,7 @@ public class AlignmentIteratorTest {
             I3G SA3G DC6
          */
 
-        AlignmentIteratorTestCase case3 = new AlignmentIteratorTestCase("I3G SA3G DC6",
+        AlignmentIteratorTestCase case3 = new AlignmentIteratorTestCase("I3G SA3G DC6", 8,
                 new int[]{0, 1, 2, 3, 3, 4, 5, 6, 7},
                 new int[]{0, 1, 2, 3, 4, 5, 6, 7, 7},
                 new int[]{0, 0, 0, 0, 1, 2, 2, 2, 3},
@@ -113,6 +113,58 @@ public class AlignmentIteratorTest {
         performTest(seq0, case3);
     }
 
+    @Test
+    public void test3() {
+        NucleotideSequence seq0 = new NucleotideSequence("ATTAGACA");
+
+        /*
+            0 ATT-AGACA 7
+               ||  || |
+            0 -TTGGGA-A 8
+
+            I3G SA3G DC6
+         */
+
+        AlignmentIteratorTestCase case3 = new AlignmentIteratorTestCase("DA0 I3G SA3G DC6", 7,
+                new int[]{0, 1, 2, 3, 3, 4, 5, 6, 7},
+                new int[]{0, 0, 1, 2, 3, 4, 5, 6, 6},
+                new int[]{0, 1, 1, 1, 2, 3, 3, 3, 4},
+                new int[]{createDeletion(0, A), NON_MUTATION, NON_MUTATION,
+                        createInsertion(3, G), createSubstitution(3, A, G),
+                        NON_MUTATION, NON_MUTATION,
+                        createDeletion(6, C),
+                        NON_MUTATION
+                });
+
+        performTest(seq0, case3);
+    }
+
+    @Test
+    public void test4() {
+        NucleotideSequence seq0 = new NucleotideSequence("ATTAGACA");
+
+        /*
+            0 ATT-AGACA 7
+              |||  |||
+            0 ATTGGGAC- 7
+
+            I3G SA3G DC6
+         */
+
+        AlignmentIteratorTestCase case3 = new AlignmentIteratorTestCase("I3G SA3G DA7", 8,
+                new int[]{0, 1, 2, 3, 3, 4, 5, 6, 7},
+                new int[]{0, 1, 2, 3, 4, 5, 6, 7, 8},
+                new int[]{0, 0, 0, 0, 1, 2, 2, 2, 2},
+                new int[]{NON_MUTATION, NON_MUTATION, NON_MUTATION,
+                        createInsertion(3, G), createSubstitution(3, A, G),
+                        NON_MUTATION, NON_MUTATION,
+                        NON_MUTATION,
+                        createDeletion(7, A)
+                });
+
+        performTest(seq0, case3);
+    }
+
     public static void performTest(NucleotideSequence seq1, AlignmentIteratorTestCase testCase) {
         Assert.assertTrue(testCase.alignment.isCompatibleWith(seq1));
 
@@ -132,7 +184,7 @@ public class AlignmentIteratorTest {
 
         i = testCase.currentMutation.length - 1;
         AlignmentIteratorReverse<NucleotideSequence> itr = new AlignmentIteratorReverse<>(testCase.alignment,
-                new Range(0, seq1.size()), testCase.seq2Position[i] + 1);
+                new Range(0, seq1.size()), testCase.seq2Size);
 
         while (itr.advance()) {
             Assert.assertEquals(testCase.seq1Position[i], itr.getSeq1Position());
@@ -146,15 +198,17 @@ public class AlignmentIteratorTest {
 
     static final class AlignmentIteratorTestCase {
         final Mutations<NucleotideSequence> alignment;
+        final int seq2Size;
         final int[] seq1Position;
         final int[] seq2Position;
         final int[] mutationsPointer;
         final int[] currentMutation;
 
-        public AlignmentIteratorTestCase(String alignment,
+        public AlignmentIteratorTestCase(String alignment, int seq2Size,
                                          int[] seq1Position, int[] seq2Position,
                                          int[] mutationsPointer, int[] currentMutation) {
             this.alignment = Mutations.decode(alignment, NucleotideSequence.ALPHABET);
+            this.seq2Size = seq2Size;
             this.seq1Position = seq1Position;
             this.seq2Position = seq2Position;
             this.mutationsPointer = mutationsPointer;
