@@ -15,9 +15,12 @@
  */
 package com.milaboratory.core.alignment;
 
+import com.milaboratory.core.Range;
 import com.milaboratory.core.mutations.Mutation;
 import com.milaboratory.core.mutations.Mutations;
+import com.milaboratory.core.sequence.NucleotideAlphabet;
 import com.milaboratory.core.sequence.NucleotideSequence;
+import com.milaboratory.core.sequence.SequenceQuality;
 import org.junit.Assert;
 import org.junit.Test;
 
@@ -41,6 +44,21 @@ public class AlignmentUtilsTest {
         Assert.assertEquals(6, (int) score);
     }
 
+    @Test
+    public void testQScore() {
+        NucleotideSequence seq1 = new NucleotideSequence("CGTA");
+        SequenceQuality seq2Quality = new SequenceQuality(new byte[]{2, 4, 5});
+
+        int[] mut = new int[2];
+        mut[0] = Mutation.createDeletion(0, NucleotideAlphabet.C);
+        mut[1] = Mutation.createSubstitution(3, NucleotideAlphabet.A, NucleotideAlphabet.G);
+        int score = AlignmentUtils.calculateQScore(seq1, seq2Quality, new Range(0, 4), new Range(0, 3),
+                new Mutations(NucleotideSequence.ALPHABET, mut),
+                LinearGapAlignmentScoring.getNucleotideBLASTScoring());
+
+        Assert.assertEquals(2 * -5 + 2 * 5 + 4 * 5 + 5 * -4, score);
+    }
+
     @Test
     public void testAffineScore() throws Exception {
         AffineGapAlignmentScoring<NucleotideSequence> scoring =
