@@ -1,5 +1,5 @@
 /*
- * Copyright 2015 MiLaboratory.com
+ * Copyright (c) 2022 MiLaboratories Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -19,10 +19,12 @@ import com.milaboratory.core.Range;
 import com.milaboratory.core.sequence.NucleotideSequence;
 import org.apache.commons.math3.random.RandomDataGenerator;
 import org.apache.commons.math3.random.Well19937c;
+import org.junit.Assert;
 import org.junit.Test;
 
 import static com.milaboratory.test.TestUtil.its;
 import static com.milaboratory.test.TestUtil.randomSequence;
+import static java.lang.Math.min;
 import static org.junit.Assert.assertEquals;
 import static org.junit.Assert.assertTrue;
 
@@ -33,46 +35,107 @@ public class BandedLinearAlignerTest {
         NucleotideSequence sequence1 = new NucleotideSequence("ATTAGACA");
         NucleotideSequence sequence2 = new NucleotideSequence("ATTGACA");
         Alignment<NucleotideSequence> alignment = BandedLinearAligner.align(LinearGapAlignmentScoring.getNucleotideBLASTScoring(), sequence1, sequence2, 0);
-        AlignerTest.assertAlignment(alignment, sequence2);
+        AlignmentTestUtils.assertAlignment(alignment, sequence2);
         //Mutations.printAlignment(sequence1, mut);
 
         sequence1 = new NucleotideSequence("ATTGACA");
         sequence2 = new NucleotideSequence("ATTAGACA");
         alignment = BandedLinearAligner.align(LinearGapAlignmentScoring.getNucleotideBLASTScoring(), sequence1, sequence2, 0);
-        AlignerTest.assertAlignment(alignment, sequence2);
+        AlignmentTestUtils.assertAlignment(alignment, sequence2);
         //Mutations.printAlignment(sequence1, mut);
 
         sequence1 = new NucleotideSequence("ATTGACA");
         sequence2 = new NucleotideSequence("AGTAGCCA");
         alignment = BandedLinearAligner.align(LinearGapAlignmentScoring.getNucleotideBLASTScoring(), sequence1, sequence2, 0);
-        AlignerTest.assertAlignment(alignment, sequence2);
+        AlignmentTestUtils.assertAlignment(alignment, sequence2);
         //Mutations.printAlignment(sequence1, mut);
 
         sequence1 = new NucleotideSequence("ATTGACAATTGACA");
         sequence2 = new NucleotideSequence("ATTGACATTGAA");
         alignment = BandedLinearAligner.align(LinearGapAlignmentScoring.getNucleotideBLASTScoring(), sequence1, sequence2, 0);
-        AlignerTest.assertAlignment(alignment, sequence2);
+        AlignmentTestUtils.assertAlignment(alignment, sequence2);
         //Mutations.printAlignment(sequence1, mut);
 
         sequence1 = new NucleotideSequence("ATTGACAATTGACA");
         sequence2 = new NucleotideSequence("ATGAAATTGCACA");
         alignment = BandedLinearAligner.align(LinearGapAlignmentScoring.getNucleotideBLASTScoring(), sequence1, sequence2, 1);
-        AlignerTest.assertAlignment(alignment, sequence2);
+        AlignmentTestUtils.assertAlignment(alignment, sequence2);
         //Mutations.printAlignment(sequence1, mut);
     }
 
     @Test
-    public void testRandomGlobal() throws Exception {
+    public void testGlobal1() {
+        NucleotideSequence sequence1 = new NucleotideSequence("AAATTGACAGA");
+        NucleotideSequence sequence2 = new NucleotideSequence(" TATTGACT ".trim());
+        Alignment<NucleotideSequence> alignment0 = BandedLinearAligner.alignAdded(
+                LinearGapAlignmentScoring.getNucleotideBLASTScoring(),
+                sequence1, sequence2,
+                5, 2,
+                1, 2, 0, 0);
+        Assert.assertEquals(new Range(1, 9), alignment0.getSequence1Range());
+        Assert.assertEquals(new Range(0, 8), alignment0.getSequence2Range());
+        System.out.println(alignment0.getAlignmentHelper());
+    }
+
+    @Test
+    public void testGlobal2() {
+        NucleotideSequence sequence1 = new NucleotideSequence("AAATTGACAGA");
+        NucleotideSequence sequence2 = new NucleotideSequence(" TATTGACG ".trim());
+        Alignment<NucleotideSequence> alignment0 = BandedLinearAligner.alignAdded(
+                LinearGapAlignmentScoring.getNucleotideBLASTScoring(),
+                sequence1, sequence2,
+                5, 2,
+                1, 2, 0, 0);
+        Assert.assertEquals(new Range(1, 10), alignment0.getSequence1Range());
+        Assert.assertEquals(new Range(0, 8), alignment0.getSequence2Range());
+        System.out.println(alignment0.getAlignmentHelper());
+    }
+
+    @Test
+    public void testGlobal3() {
+        NucleotideSequence sequence1 = new NucleotideSequence(" TATTGACT ".trim());
+        NucleotideSequence sequence2 = new NucleotideSequence("AAATTGACAGA");
+        Alignment<NucleotideSequence> alignment0 = BandedLinearAligner.alignAdded(
+                LinearGapAlignmentScoring.getNucleotideBLASTScoring(),
+                sequence1, sequence2,
+                2, 5,
+                0, 0, 1, 2);
+        Assert.assertEquals(new Range(0, 8), alignment0.getSequence1Range());
+        Assert.assertEquals(new Range(1, 9), alignment0.getSequence2Range());
+        System.out.println(alignment0.getAlignmentHelper());
+    }
+
+    @Test
+    public void testGlobal4() {
+        NucleotideSequence sequence1 = new NucleotideSequence(" TATTGACG ".trim());
+        NucleotideSequence sequence2 = new NucleotideSequence("AAATTGACAGA");
+        Alignment<NucleotideSequence> alignment0 = BandedLinearAligner.alignAdded(
+                LinearGapAlignmentScoring.getNucleotideBLASTScoring(),
+                sequence1, sequence2,
+                2, 5,
+                0, 0, 1, 2);
+        Assert.assertEquals(new Range(0, 8), alignment0.getSequence1Range());
+        Assert.assertEquals(new Range(1, 10), alignment0.getSequence2Range());
+        System.out.println(alignment0.getAlignmentHelper());
+    }
+
+    @Test
+    public void testRandomGlobal() {
         int its = its(100, 10000);
         RandomDataGenerator random = new RandomDataGenerator(new Well19937c());
         for (int i = 0; i < its; ++i) {
             NucleotideSequence seq1, seq2;
             seq1 = randomSequence(NucleotideSequence.ALPHABET, random, 10, 100);
             seq2 = randomSequence(NucleotideSequence.ALPHABET, random, 10, 100);
-            Alignment<NucleotideSequence> alignment = BandedLinearAligner.align(LinearGapAlignmentScoring.getNucleotideBLASTScoring(),
+            int w1 = random.nextInt(1, min(seq1.size(), seq2.size()) - 1);
+            int w2 = random.nextInt(1, min(seq1.size(), seq2.size()) - 1);
+            Alignment<NucleotideSequence> alignment = BandedLinearAligner.alignAdded(LinearGapAlignmentScoring.getNucleotideBLASTScoring(),
                     seq1, seq2,
-                    random.nextInt(0, Math.min(seq1.size(), seq2.size()) - 1));
-            AlignerTest.assertAlignment(alignment, seq2);
+                    Math.max(0, seq1.size() - seq2.size()) + w1,
+                    Math.max(0, seq2.size() - seq1.size()) + w2,
+                    random.nextInt(0, min(2, w1 - 1)), random.nextInt(0, min(2, w1 - 1)),
+                    random.nextInt(0, min(2, w2 - 1)), random.nextInt(0, min(2, w2 - 1)));
+            AlignmentTestUtils.assertAlignment(alignment, seq2);
         }
     }
 
@@ -80,10 +143,9 @@ public class BandedLinearAlignerTest {
     public void test5() {
         NucleotideSequence seq1 = new NucleotideSequence("A"), seq2 = new NucleotideSequence("ATTA");
         Alignment<NucleotideSequence> alignment = BandedLinearAligner.align(LinearGapAlignmentScoring.getNucleotideBLASTScoring(), seq1, seq2, 1);
-        AlignerTest.assertAlignment(alignment, seq2);
+        AlignmentTestUtils.assertAlignment(alignment, seq2);
     }
 
-
     @Test
     public void testRandomLeft() throws Exception {
         int its = its(100, 10000);
@@ -94,9 +156,9 @@ public class BandedLinearAlignerTest {
             seq2 = randomSequence(NucleotideSequence.ALPHABET, random, 80, 84);
             Alignment<NucleotideSequence> r = BandedLinearAligner.alignSemiLocalLeft(LinearGapAlignmentScoring.getNucleotideBLASTScoring(),
                     seq1, seq2,
-                    random.nextInt(0, Math.min(seq1.size(), seq2.size()) - 1),
+                    random.nextInt(0, min(seq1.size(), seq2.size()) - 1),
                     -10);
-            AlignerTest.assertAlignment(r, seq2);
+            AlignmentTestUtils.assertAlignment(r, seq2);
         }
     }
 
@@ -111,9 +173,9 @@ public class BandedLinearAlignerTest {
             Alignment<NucleotideSequence> r =
                     BandedLinearAligner.alignSemiLocalRight(LinearGapAlignmentScoring.getNucleotideBLASTScoring(),
                             seq1, seq2,
-                            random.nextInt(0, Math.min(seq1.size(), seq2.size()) - 1),
+                            random.nextInt(0, min(seq1.size(), seq2.size()) - 1),
                             -10);
-            AlignerTest.assertAlignment(r, seq2);
+            AlignmentTestUtils.assertAlignment(r, seq2);
         }
     }
 
@@ -122,7 +184,7 @@ public class BandedLinearAlignerTest {
         NucleotideSequence seq1 = new NucleotideSequence("ATTAGACA");
         NucleotideSequence seq2 = new NucleotideSequence("ATTACGC");
         Alignment<NucleotideSequence> r = BandedLinearAligner.alignSemiLocalLeft(LinearGapAlignmentScoring.getNucleotideBLASTScoring(), seq1, seq2, 0, -10);
-        AlignerTest.assertAlignment(r, seq2);
+        AlignmentTestUtils.assertAlignment(r, seq2);
         assertEquals(4, r.getSequence1Range().getTo());
         assertEquals(4, r.getSequence2Range().getTo());
 
@@ -130,7 +192,7 @@ public class BandedLinearAlignerTest {
         seq2 = new NucleotideSequence("ATTGACGC");
         r = BandedLinearAligner.alignSemiLocalLeft(LinearGapAlignmentScoring.getNucleotideBLASTScoring(), seq1, seq2, 1, -10);
         //Mutations.printAlignment(seq1.getSubSequence(0, r.sequence1Stop + 1), r.mutations);
-        AlignerTest.assertAlignment(r, seq2);
+        AlignmentTestUtils.assertAlignment(r, seq2);
         assertEquals(7, r.getSequence1Range().getTo());
         assertEquals(6, r.getSequence2Range().getTo());
 
@@ -138,7 +200,7 @@ public class BandedLinearAlignerTest {
         seq2 = new NucleotideSequence("ATTAGACATTAGACA");
         r = BandedLinearAligner.alignSemiLocalLeft(LinearGapAlignmentScoring.getNucleotideBLASTScoring(), seq1, seq2, 1, -10);
         //Mutations.printAlignment(seq1.getSubSequence(0, r.sequence1Stop + 1), r.mutations);
-        AlignerTest.assertAlignment(r, seq2);
+        AlignmentTestUtils.assertAlignment(r, seq2);
         assertEquals(7, r.getSequence1Range().getTo());
         assertEquals(8, r.getSequence2Range().getTo());
     }
@@ -149,7 +211,7 @@ public class BandedLinearAlignerTest {
         NucleotideSequence seq2 = new NucleotideSequence("GACA");
         Alignment<NucleotideSequence> r = BandedLinearAligner.alignSemiLocalRight(LinearGapAlignmentScoring.getNucleotideBLASTScoring(), seq1, seq2, 0, -10);
         //Mutations.printAlignment(seq1.getSubSequence(r.sequence1Stop, seq1.size()), r.mutations);
-        AlignerTest.assertAlignment(r, seq2);
+        AlignmentTestUtils.assertAlignment(r, seq2);
         assertEquals(4, r.getSequence1Range().getFrom());
         assertEquals(0, r.getSequence2Range().getFrom());
 
@@ -157,7 +219,7 @@ public class BandedLinearAlignerTest {
         seq2 = new NucleotideSequence("GCGAATAGACA");
         r = BandedLinearAligner.alignSemiLocalRight(LinearGapAlignmentScoring.getNucleotideBLASTScoring(), seq1, seq2, 0, -10);
         //Mutations.printAlignment(seq1.getSubSequence(r.sequence1Stop, seq1.size()), Mutations.move(r.mutations, -r.sequence1Stop));
-        AlignerTest.assertAlignment(r, seq2);
+        AlignmentTestUtils.assertAlignment(r, seq2);
         assertEquals(7, r.getSequence1Range().getFrom());
         assertEquals(3, r.getSequence2Range().getFrom());
     }
@@ -170,7 +232,7 @@ public class BandedLinearAlignerTest {
                 seq1, seq2, 0, seq1.size(), 0, 0, seq2.size(), 0, 1);
         assertEquals(seq1.size(), la.getSequence1Range().getTo());
         assertEquals(seq2.size(), la.getSequence2Range().getTo());
-        AlignerTest.assertAlignment(la, seq2);
+        AlignmentTestUtils.assertAlignment(la, seq2);
     }
 
     @Test
@@ -181,7 +243,7 @@ public class BandedLinearAlignerTest {
                 seq1, seq2, 0, seq1.size(), 1, 0, seq2.size(), 1, 1);
         assertEquals(seq1.size(), la.getSequence1Range().getTo());
         assertEquals(seq2.size() - 1, la.getSequence2Range().getTo());
-        AlignerTest.assertAlignment(la, seq2);
+        AlignmentTestUtils.assertAlignment(la, seq2);
     }
 
     @Test
@@ -192,7 +254,7 @@ public class BandedLinearAlignerTest {
                 seq1, seq2, 0, seq1.size(), 1, 0, seq2.size(), 0, 1);
         assertEquals(seq1.size(), la.getSequence1Range().getTo());
         assertEquals(seq2.size(), la.getSequence2Range().getTo());
-        AlignerTest.assertAlignment(la, seq2);
+        AlignmentTestUtils.assertAlignment(la, seq2);
     }
 
     @Test
@@ -220,7 +282,7 @@ public class BandedLinearAlignerTest {
             assertTrue(la.getSequence1Range().getTo() >= offset1 + length1 - added1);
             assertTrue(la.getSequence2Range().getTo() >= offset2 + length2 - added2);
 
-            AlignerTest.assertAlignment(la, seq2);
+            AlignmentTestUtils.assertAlignment(la, seq2);
         }
     }
 
@@ -232,7 +294,7 @@ public class BandedLinearAlignerTest {
                 seq1, seq2, 0, seq1.size(), 0, 0, seq2.size(), 0, 1);
         assertEquals(0, la.getSequence1Range().getFrom());
         assertEquals(0, la.getSequence2Range().getFrom());
-        AlignerTest.assertAlignment(la, seq2);
+        AlignmentTestUtils.assertAlignment(la, seq2);
     }
 
     @Test
@@ -243,7 +305,7 @@ public class BandedLinearAlignerTest {
                 seq1, seq2, 0, seq1.size(), 0, 0, seq2.size(), 1, 1);
         assertEquals(0, la.getSequence1Range().getFrom());
         assertEquals(0, la.getSequence2Range().getFrom());
-        AlignerTest.assertAlignment(la, seq2);
+        AlignmentTestUtils.assertAlignment(la, seq2);
     }
 
     @Test
@@ -254,7 +316,7 @@ public class BandedLinearAlignerTest {
                 seq1, seq2, 0, seq1.size(), 0, 0, seq2.size(), 3, 1);
         assertEquals(0, la.getSequence1Range().getFrom());
         assertEquals(2, la.getSequence2Range().getFrom());
-        AlignerTest.assertAlignment(la, seq2);
+        AlignmentTestUtils.assertAlignment(la, seq2);
     }
 
     @Test
@@ -301,7 +363,86 @@ public class BandedLinearAlignerTest {
             assertTrue(la.getSequence1Range().getFrom() <= offset1 + added1);
             assertTrue(la.getSequence2Range().getFrom() <= offset2 + added2);
 
-            AlignerTest.assertAlignment(la, seq2);
+            AlignmentTestUtils.assertAlignment(la, seq2);
         }
     }
+
+    @Test
+    public void testCase1() {
+        NucleotideSequence seq1 = new NucleotideSequence("GCGTGAAGACTGCAGGCATTGAGTACGTTACTAGTCCAGTGGGGCCCAACCGTAACATTGCGTGTGACTGGTTGCTTAGCGGGTGACGGCGTTTCAGGTTACGCCCTCTGTGCATCACCGATAGCGTTGTTTGAGCTTTCTAACGTCTCAAACCTGCTTGCGCCCTACGGTTTCTCTATATATACACGAAAGGGGCTGGTCGCAACCTCGCAATTCTATCCCTCCAACCTCCCTCGAATTTTT");
+        NucleotideSequence seq2 = new NucleotideSequence("ATAGTAAAGCGTGAAGACTTGCAGGCATTGGTACGTTATTAGTCCAGTGGGGCCACAACCGTAACATTGCGTGTGACTGGTGCTTAGCGGGTGACGGCGTTCAGGTTACGCCCTCTGTGCATCACCGATTAGCGTTGTCTGAGCTTTCTAACGTCTCAAACCTGCTTGCGCCCTACGGTTTCTCTATAGTATCACGAAAGGGTCTGGTCGCAACCTCGCAATTCTATCCCTCCAACCTCCCTTGAATTTTTCTACCC");
+        System.out.println(seq1.getRange(0, 243));
+        System.out.println(seq2.getRange(5, 5 + 249));
+        Alignment<NucleotideSequence> al = BandedLinearAligner.alignAdded(LinearGapAlignmentScoring.getNucleotideBLASTScoring(), seq1, seq2,
+                0, 243, 5, 249,
+                4, 7,
+                0, 0, 6, 6);
+        AlignmentTestUtils.assertAlignment(al, seq2);
+    }
+
+    @Test
+    public void testCase2() {
+        NucleotideSequence seq1 = new NucleotideSequence("");
+        NucleotideSequence seq2 = new NucleotideSequence("A");
+        Alignment<NucleotideSequence> al = BandedLinearAligner.alignAdded(LinearGapAlignmentScoring.getNucleotideBLASTScoring(), seq1, seq2,
+                0, 0, 0, 1,
+                2, 3,
+                0, 0, 0, 1);
+        AlignmentTestUtils.assertAlignment(al, seq2);
+    }
+
+    @Test
+    public void testCase3() {
+        NucleotideSequence seq1 = new NucleotideSequence("");
+        NucleotideSequence seq2 = new NucleotideSequence("A");
+        Alignment<NucleotideSequence> al = BandedLinearAligner.alignAdded(LinearGapAlignmentScoring.getNucleotideBLASTScoring(), seq1, seq2,
+                0, 0, 0, 1,
+                2, 3,
+                0, 0, 0, 0);
+        AlignmentTestUtils.assertAlignment(al, seq2);
+    }
+
+    @Test
+    public void testCase4() {
+        NucleotideSequence seq1 = new NucleotideSequence("");
+        NucleotideSequence seq2 = new NucleotideSequence("A");
+        Alignment<NucleotideSequence> al = BandedLinearAligner.alignAdded(LinearGapAlignmentScoring.getNucleotideBLASTScoring(), seq1, seq2,
+                0, 0, 0, 1,
+                2, 3,
+                0, 0, 1, 0);
+        AlignmentTestUtils.assertAlignment(al, seq2);
+    }
+
+    @Test
+    public void testCase5() {
+        NucleotideSequence seq1 = new NucleotideSequence("A");
+        NucleotideSequence seq2 = new NucleotideSequence("");
+        Alignment<NucleotideSequence> al = BandedLinearAligner.alignAdded(LinearGapAlignmentScoring.getNucleotideBLASTScoring(), seq1, seq2,
+                0, 1, 0, 0,
+                2, 3,
+                0, 1, 0, 0);
+        AlignmentTestUtils.assertAlignment(al, seq2);
+    }
+
+    @Test
+    public void testCase6() {
+        NucleotideSequence seq1 = new NucleotideSequence("A");
+        NucleotideSequence seq2 = new NucleotideSequence("");
+        Alignment<NucleotideSequence> al = BandedLinearAligner.alignAdded(LinearGapAlignmentScoring.getNucleotideBLASTScoring(), seq1, seq2,
+                0, 1, 0, 0,
+                2, 3,
+                0, 0, 0, 0);
+        AlignmentTestUtils.assertAlignment(al, seq2);
+    }
+
+    @Test
+    public void testCase7() {
+        NucleotideSequence seq1 = new NucleotideSequence("A");
+        NucleotideSequence seq2 = new NucleotideSequence("");
+        Alignment<NucleotideSequence> al = BandedLinearAligner.alignAdded(LinearGapAlignmentScoring.getNucleotideBLASTScoring(), seq1, seq2,
+                0, 1, 0, 0,
+                2, 3,
+                1, 0, 0, 0);
+        AlignmentTestUtils.assertAlignment(al, seq2);
+    }
 }
