@@ -24,7 +24,7 @@ public class BlastTest {
     static final AtomicBoolean first = new AtomicBoolean(true);
 
     @Before
-    public void setUp() throws Exception {
+    public void setUp() {
         Assume.assumeTrue(Blast.isBlastAvailable());
         if (first.compareAndSet(true, false))
             System.out.println("Blast: " + Blast.getBlastCommand(Blast.CMD_BLASTN, false));
