@@ -1,5 +1,5 @@
 /*
- * Copyright 2015 MiLaboratory.com
+ * Copyright (c) 2022 MiLaboratories Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -36,7 +36,7 @@ import static com.milaboratory.core.mutations.generator.MutationsGenerator.gener
 import static com.milaboratory.core.sequence.NucleotideSequence.EMPTY;
 import static com.milaboratory.test.TestUtil.*;
 
-public class KAlignerTest extends AlignmentTest {
+public class KAlignerTest extends AbstractAlignmentTest {
     public static final KAlignerParameters gParams = new KAlignerParameters(5, false, false,
             1.5f, 0.75f, 1.0f, -0.1f, -0.3f, 4, 10, 15, 2, -10,
             40.0f, 0.87f, 7,
