@@ -16,8 +16,7 @@
 package com.milaboratory.core.merger;
 
 import com.milaboratory.core.PairedEndReadsLayout;
-import com.milaboratory.core.io.sequence.PairedRead;
-import com.milaboratory.core.io.sequence.SingleReadImpl;
+import com.milaboratory.core.sequence.NSQTuple;
 import com.milaboratory.core.sequence.NSequenceWithQuality;
 import com.milaboratory.core.sequence.SequenceQuality;
 import org.junit.Assert;
@@ -158,9 +157,9 @@ public class MismatchOnlyPairedReadMergerTest {
         MismatchOnlyPairedReadMerger merger = new MismatchOnlyPairedReadMerger(overlap, 1.0 - 1.0 * maxMuts / overlap, 55,
                 mergingAlgorithm,
                 PairedEndReadsLayout.Collinear);
-        PairedReadMergingResult processed = merger.process(new PairedRead(
-                new SingleReadImpl(0, new NSequenceWithQuality(seq1, lets('A', seq1.length())), "A"),
-                new SingleReadImpl(0, new NSequenceWithQuality(seq2, lets('B', seq2.length())), "B")));
+        PairedReadMergingResult processed = merger.process(new NSQTuple(0,
+                new NSequenceWithQuality(seq1, lets('A', seq1.length())),
+                new NSequenceWithQuality(seq2, lets('B', seq2.length()))));
         if (expectedSequence == null)
             Assert.assertFalse(processed.isSuccessful());
         else {
@@ -172,9 +171,9 @@ public class MismatchOnlyPairedReadMergerTest {
         merger = new MismatchOnlyPairedReadMerger(overlap, 1.0 - 1.0 * maxMuts / overlap, 55,
                 mergingAlgorithm,
                 PairedEndReadsLayout.Unknown);
-        processed = merger.process(new PairedRead(
-                new SingleReadImpl(0, new NSequenceWithQuality(seq1, lets('A', seq1.length())), "A"),
-                new SingleReadImpl(0, new NSequenceWithQuality(seq2, lets('B', seq2.length())).getReverseComplement(), "B")));
+        processed = merger.process(new NSQTuple(0,
+                new NSequenceWithQuality(seq1, lets('A', seq1.length())),
+                new NSequenceWithQuality(seq2, lets('B', seq2.length())).getReverseComplement()));
         if (expectedSequence == null)
             Assert.assertFalse(processed.isSuccessful());
         else {
