@@ -72,13 +72,7 @@ public class MotifTest {
                 assertTrue(motif.matches(seq, i));
         }
     }
-
-    @Test
-    public void test4() throws Exception {
-        Motif<AminoAcidSequence> se = new AminoAcidSequence("CASSLAP").toMotif();
-        IOTestUtil.assertJavaSerialization(se);
-    }
-
+    
     @Test
     public void testInformation() {
         assertEquals(20, new NucleotideSequence("ATTAGACAAT").toMotif().matchBitScore(), 0.1);
