@@ -1,5 +1,5 @@
 /*
- * Copyright 2015 MiLaboratory.com
+ * Copyright (c) 2022 MiLaboratories Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -78,6 +78,7 @@ public class SequenceTreeMapTest {
             node = iterator.next();
             assertTrue(ints.remove(node.object));
             assertTrue(sequences.remove(iterator.getSequence()));
+            map.remove((NucleotideSequence) iterator.getSequence());
         }
 
         assertTrue(sequences.isEmpty());
@@ -494,12 +495,12 @@ public class SequenceTreeMapTest {
     /*
      * Utility functions and their tests
      */
-//    private Sequence getRandomSequence(Alphabet alphabet, int length) {
-//        SequenceBuilder builder = alphabet.getBuilderFactory().create(length);
-//        for (int i = 0; i < length; ++i)
-//            builder.setCode(i, (byte) random.nextInt(alphabet.size()));
-//        return builder.create();
-//    }
+    //    private Sequence getRandomSequence(Alphabet alphabet, int length) {
+    //        SequenceBuilder builder = alphabet.getBuilderFactory().create(length);
+    //        for (int i = 0; i < length; ++i)
+    //            builder.setCode(i, (byte) random.nextInt(alphabet.size()));
+    //        return builder.create();
+    //    }
 
     private Sequence introduceMutation(Sequence sequence, int type) {
         SequenceBuilder builder;
@@ -795,9 +796,12 @@ public class SequenceTreeMapTest {
 
                 while ((n = neighborhoodIterator.nextNode()) != null) {
                     Mutations<NucleotideSequence> mutations = neighborhoodIterator.getCurrentMutations();
-                    assertEquals(n.getObject(), mutations.mutate(clusters[i].sequence));
+                    NucleotideSequence seq = n.getObject();
+                    assertEquals(seq, mutations.mutate(clusters[i].sequence));
+                    sequenceTreeMap.remove(seq); // testing concurrent modification of the tree
                 }
             }
+            assertFalse(sequenceTreeMap.values().iterator().hasNext());
         }
     }
 
