@@ -1,5 +1,5 @@
 /*
- * Copyright 2015 MiLaboratory.com
+ * Copyright (c) 2022 MiLaboratories Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -23,7 +23,7 @@ public class TestJsonClass1Test {
     @Test
     public void test1() throws Exception {
         TestJsonClass1 c1 = new TestJsonClass1(1, "FER");
-        String str = GlobalObjectMappers.ONE_LINE.writeValueAsString(c1);
-        Assert.assertEquals(c1, GlobalObjectMappers.ONE_LINE.readValue(str, TestJsonClass1.class));
+        String str = GlobalObjectMappers.getOneLine().writeValueAsString(c1);
+        Assert.assertEquals(c1, GlobalObjectMappers.getOneLine().readValue(str, TestJsonClass1.class));
     }
 }
\ No newline at end of file
